@@ -82,9 +82,10 @@ class AutoInc_Harness(object):
                         if model_name + "(" in line or \
                             (model_name + "." in line and line.find(model_name) < line.find(".") and "(" in line):
                             to_transform = True
-                    if not to_transform:
+                    if not to_transform and domain_ == 'onnx':
+                        pass
+                    elif not to_transform:
                         continue
-
                     ### information
 
                     # search DataLoader definition in this file
@@ -142,7 +143,7 @@ class AutoInc_Harness(object):
                     ### check
 
                     bk_trans_content_this = bk_trans_content[bk_trans_location.index(loc)]
-                    if file_path_idx == 0 and (domain_ == 'transformers_trainer' or domain_ == 'torchvision'):
+                    if file_path_idx == 0 and (domain_ in ['transformers_trainer', 'torchvision', 'onnx']):
                         pass
                     elif ("INPUT_NAME" in bk_trans_content_this and input_name == "") \
                             or ("DATALOADER_NAME" in bk_trans_content_this and dataloader_name == "") \
@@ -199,10 +200,12 @@ class AutoInc_Harness(object):
                                     trans_insert_location = i + i_search
                         trans_insert_location = min(max(trans_insert_location, put_below_idx), put_above_idx)
                     elif file_path_idx == 0 and domain_ == 'torchvision':
+                        trans_insert_location = 1
                         for i in range(len(lines)):
                             line = lines[i]
-                            if "val_loader" in line and ("torch.utils.data.DataLoader" in line \
-                                or "utils.data.DataLoader" in line or "DataLoader" in line):
+                            if "val_loader" in line and "aux_val_loader" not in line \
+                                and ("torch.utils.data.DataLoader" in line \
+                                    or "utils.data.DataLoader" in line or "DataLoader" in line):
                                 torchvision_indent = get_line_indent_level(line)
                                 if "(" in line and line.count(")") == line.count("("):
                                     trans_insert_location = i + 1
@@ -260,6 +263,10 @@ class AutoInc_Harness(object):
                         lines_to_insert = lines_to_insert \
                             .replace("EVAL_FUNC_LINES", globals.list_eval_func_lines[0]) \
                             .replace("DATALOADER_NAME", globals.list_calib_dataloader_name[0])
+                    elif domain_ =='onnx':
+                        lines_to_insert = lines_to_insert \
+                            .replace("EVAL_FUNCTION_NAME", globals.list_eval_func_name[0]) \
+                            .replace("DATALOADER_NAME", globals.list_calib_dataloader_name[0])
                     else:
                         lines_to_insert = lines_to_insert \
                             .replace("DATALOADER_NAME", dataloader_name)
@@ -268,8 +275,8 @@ class AutoInc_Harness(object):
                     lines_to_insert = lines_to_insert.replace(
                         "[+] ", " " * insert_indent_level)
                     # add begin indicator
-                    lines_to_insert = " " * insert_indent_level + "# [coder] " + \
-                        self.backend + " for " + model_name + " [begin]\n" + lines_to_insert
+                    lines_to_insert = " " * insert_indent_level + "# [NeuralCoder] " + \
+                        self.backend + " for " + model_name + " [Beginning Line]\n" + lines_to_insert
                     # replace INDICATIONS with real stuff
                     lines_to_insert = lines_to_insert \
                         .replace("MODEL_NAME", model_name) \
@@ -278,8 +285,8 @@ class AutoInc_Harness(object):
                         .replace("\n", " # [coder-enabled]\n")
                     # add end indicator
                     lines_to_insert += " # [coder-enabled]\n" + \
-                        " " * insert_indent_level + "# [coder] " + self.backend + " for " + \
-                        model_name + " [end] # [coder-enabled]"
+                        " " * insert_indent_level + "# [NeuralCoder] " + self.backend + " for " + \
+                        model_name + " [Ending Line] # [coder-enabled]"
 
                     ### register
 
@@ -370,8 +377,8 @@ class AutoInc_Harness(object):
                                     lines_to_insert = lines_to_insert.replace(
                                         "[+] ", " " * insert_indent_level)
                                     # add begin indicator
-                                    lines_to_insert = " " * insert_indent_level + "# [coder] " + \
-                                        self.backend + " [begin] \n" + lines_to_insert
+                                    lines_to_insert = " " * insert_indent_level + "# [NeuralCoder] " + \
+                                        self.backend + " [Beginning Line] \n" + lines_to_insert
                                     # replace INDICATIONS with real stuff 
                                     # (for now, inference_line related transformations )
                                     # (have nothing to do with input, dataloader etc, )
@@ -379,8 +386,8 @@ class AutoInc_Harness(object):
                                     lines_to_insert = lines_to_insert.replace("\n", " # [coder-enabled]\n")
                                     # add end indicator
                                     lines_to_insert += " # [coder-enabled]\n" + \
-                                        " " * insert_indent_level + "# [coder] " + \
-                                        self.backend + " [end] # [coder-enabled]"
+                                        " " * insert_indent_level + "# [NeuralCoder] " + \
+                                        self.backend + " [Ending Line] # [coder-enabled]"
 
                                     # customized argument
                                     if self.backend == "pytorch_benchmark":
