@@ -116,6 +116,15 @@ class Harness(object):
                                     input_def_line_indent_level = get_line_indent_level(line)
                                     input_def_line_idx = i
                     
+                    # search trainer definition in this file (for transformers trainer only)
+                    trainer_def_line_idx = -1
+                    for i in range(len(lines)):
+                        line = lines[i]
+                        if not single_line_comment_or_empty_line_detection(line):
+                            if "trainer = Trainer(" in line:
+                                trainer_def_line_indent_level = get_line_indent_level(line)
+                                trainer_def_line_idx = i
+
                     # serach model definition line and its end line index
                     # (only has 1 model definition line, because it's in loop of globals.list_model_def_instance)
                     for i in range(len(lines)):
@@ -180,6 +189,9 @@ class Harness(object):
                         trans_insert_location = \
                             min(max(model_definition_end_line_idx,
                                 put_below_idx), put_above_idx)
+                        if trainer_def_line_idx > 0:
+                            trans_insert_location = trainer_def_line_idx - 1
+                            # for transformers trainer to put right above trainer def
                     if "insert_below_dataloader_definition_line" in loc:
                         try:
                             dataloader_def_line_idx
@@ -204,7 +216,8 @@ class Harness(object):
                                                             put_below_idx), put_above_idx))
                     
                     insert_indent_level = get_line_indent_level(lines[trans_insert_location - 1])
-
+                    if trainer_def_line_idx > 0: # for transformers trainer to put right above trainer def
+                        insert_indent_level = get_line_indent_level(lines[trans_insert_location])
                     ### content
 
                     # lines to insert
@@ -213,8 +226,8 @@ class Harness(object):
                     lines_to_insert = lines_to_insert.replace(
                         "[+] ", " " * insert_indent_level)
                     # add begin indicator
-                    lines_to_insert = " " * insert_indent_level + "# [coder] " + \
-                        self.backend + " for " + model_name + " [begin]\n" + lines_to_insert
+                    lines_to_insert = " " * insert_indent_level + "# [NeuralCoder] " + \
+                        self.backend + " for " + model_name + " [Beginning Line]\n" + lines_to_insert
                     # replace INDICATIONS with real stuff
                     lines_to_insert = lines_to_insert \
                         .replace("MODEL_NAME", model_name) \
@@ -224,8 +237,8 @@ class Harness(object):
                         .replace("\n", " # [coder-enabled]\n")
                     # add end indicator
                     lines_to_insert += " # [coder-enabled]\n" + \
-                        " " * insert_indent_level + "# [coder] " + self.backend + " for " + \
-                        model_name + " [end] # [coder-enabled]"
+                        " " * insert_indent_level + "# [NeuralCoder] " + self.backend + " for " + \
+                        model_name + " [Ending Line] # [coder-enabled]"
 
                     ### register
 
@@ -256,7 +269,7 @@ class Harness(object):
                             if is_eval_func and "[coder-enabled]" not in line:
                                 if eval_func_type == "non-forward":
                                     pass # do something
-                                
+                                inference_line = line
                                 inference_line_indent_level = get_line_indent_level(line)
 
                                 if "indent_inference_line" in loc:
@@ -299,7 +312,20 @@ class Harness(object):
                                 if "insert_above_inference_line" in loc:
                                     idx_offset = 0
                                 elif "insert_below_inference_line" in loc:
-                                    idx_offset = 1
+                                    if ")" in line:  # e.g. model = Net(xxx)
+                                        idx_offset = 1
+                                    else:  # e.g. model = Net(xxx, \n xxx, \n xxx)
+                                        do_search = True
+                                        i_search = 1
+                                        while do_search:
+                                            following_line = lines[line_idx + i_search]
+                                            if ")" in following_line:
+                                                do_search = False
+                                            i_search += 1
+                                            inference_line = \
+                                                inference_line + "\n" + \
+                                                " " * (get_line_indent_level(line) + 4) + following_line
+                                        idx_offset = i_search
                                 
                                 if "insert_above_inference_line" in loc or "insert_below_inference_line" in loc:
                                     bk_trans_content_this = bk_trans_content[bk_trans_location.index(loc)]
@@ -316,8 +342,8 @@ class Harness(object):
                                     lines_to_insert = lines_to_insert.replace(
                                         "[+] ", " " * insert_indent_level)
                                     # add begin indicator
-                                    lines_to_insert = " " * insert_indent_level + "# [coder] " + \
-                                        self.backend + " [begin] \n" + lines_to_insert
+                                    lines_to_insert = " " * insert_indent_level + "# [NeuralCoder] " + \
+                                        self.backend + " [Beginning Line] \n" + lines_to_insert
                                     # replace INDICATIONS with real stuff 
                                     # (for now, inference_line related transformations )
                                     # (have nothing to do with input, dataloader etc, )
@@ -325,8 +351,8 @@ class Harness(object):
                                     lines_to_insert = lines_to_insert.replace("\n", " # [coder-enabled]\n")
                                     # add end indicator
                                     lines_to_insert += " # [coder-enabled]\n" + \
-                                        " " * insert_indent_level + "# [coder] " + \
-                                        self.backend + " [end] # [coder-enabled]"
+                                        " " * insert_indent_level + "# [NeuralCoder] " + \
+                                        self.backend + " [Ending Line] # [coder-enabled]"
 
                                     # customized argument
                                     if self.backend == "pytorch_benchmark":
@@ -335,7 +361,7 @@ class Harness(object):
                                         lines_to_insert = lines_to_insert.replace("ACCURACY_MODE", 
                                                                                     str(globals.eval_accuracy))
                                         lines_to_insert = lines_to_insert.replace("INFERENCE_LINE", 
-                                                                                    line.strip())
+                                                                                    inference_line.strip())
 
                                     ### register
                                     
@@ -354,6 +380,7 @@ class Harness(object):
                                         )
                                         globals.list_trans_insert_lines_to_insert[idx].append(lines_to_insert)
 
+                                break # already transformed this line, so skip any further model_name search
                         line_idx += 1
 
             # PART 3 - for customized location
