@@ -16,6 +16,7 @@ import os
 import subprocess
 import logging
 import time
+import yaml
 
 from . import globals
 
@@ -27,7 +28,7 @@ def enable(
     code,
     features,
     target_batch_size=1,  # effective for feature "pytorch_change_batch_size"
-    num_benchmark_iteration=30,  # effective for feature "pytorch_benchmark"
+    num_benchmark_iteration=10,  # effective for feature "pytorch_benchmark"
     eval_accuracy=False,
     generate_patch=True,
     overwrite=False,
@@ -39,14 +40,14 @@ def enable(
     logging_level="info",
     run_bench=False,
     entry_code="",
-    entry_code_args="",
+    args="",
     mode="throughput",
     cpu_set_env=True,
     ncore_per_instance=-1,  # only for "self_defined" mode
     ninstances=-1,  # only for "self_defined" mode
     bench_batch_size=-1,  # only for "self_defined" mode
     test_code_line=False, # print code line info for debug use
-    brutal_mode=False, # TO-ADD: return a folder with user-input code and artificial pip folders
+    cache_load_transformers=True,
 ):
     """enable a feature or a couple of features for the code
 
@@ -115,6 +116,7 @@ def enable(
         "pytorch_aliblade",
         "tensorflow_amp",
         "keras_amp",
+        "onnx_inc_static_quant_qlinear"
     ]
     '''
 
@@ -145,16 +147,18 @@ def enable(
 
     #### Feature Enabling
 
-    globals.num_benchmark_iteration = str(num_benchmark_iteration + 5)
+    globals.num_benchmark_iteration = str(num_benchmark_iteration + 10) # 10: warmup iteration number
 
     globals.eval_accuracy = eval_accuracy
 
+    globals.cache_load_transformers = cache_load_transformers
+
     # move "pytorch_benchmark" to the last
     from .utils.common import move_element_to_last
     features = move_element_to_last(features, "pytorch_benchmark")
 
     # not in harness scope
-    features_not_using_harness = [
+    features_outside_harness = [
         "pytorch_change_batch_size",
         "pytorch_cuda_to_cpu",
         "pytorch_lightning_bf16_cpu",
@@ -166,7 +170,7 @@ def enable(
     #     "pytorch_inc_static_quant_ipex" in features:
     #     features = ["pytorch_dummy_dataloader"] + features
     
-    # features that need detecting inputs first
+    # features that need reclaiming inputs first (e.g. for "for step, inputs in enumerate(dataloader)")
     if "pytorch_jit_trace" in features or \
         "pytorch_jit_trace_ofi" in features or \
         "pytorch_inc_static_quant_fx" in features or \
@@ -175,7 +179,24 @@ def enable(
 
     transformed_list_code_path = []
 
-    for feature in features:
+    ## Determine Code Domain
+    # reset globals
+    globals.reset_globals()
+
+    from .utils import handle_user_input
+    globals.list_code_path, num_user_code_path = handle_user_input.get_all_code_path(code)
+
+    from .coders.autoinc import domain
+    code_domain = domain.determine_domain(globals.list_code_path[0])
+    if code_domain == "transformers_trainer":
+        if "pytorch_benchmark" in features:
+            features = ["pytorch_reclaim_inference_transformers_trainer"] + features
+            # for BS
+            args += " --per_device_eval_batch_size " + str(target_batch_size)
+            globals.batch_size_changed = True
+
+    ## Feature Transformation
+    for idx_feature, feature in enumerate(features):
 
         # reset globals
         globals.reset_globals()
@@ -199,7 +220,7 @@ def enable(
             list_transformed_code.append(open(i, 'r').read())
 
         ## 1. Features in Harness Scope
-        if feature not in features_not_using_harness:
+        if feature not in features_outside_harness:
             from .graphers.code_line import register_code_line
             from .graphers.model import register_nnModule_class, register_nnModule_instance_definition
             from .graphers.function import register_func_wrap_pair
@@ -209,20 +230,30 @@ def enable(
             register_code_line()
             register_func_wrap_pair()
             register_nnModule_class()
+            if cache_load_transformers:
+                preload_file = open(os.path.dirname(__file__) +
+                    "/graphers/preloads/" + "transformers" + ".yaml")
+                preload_dict = yaml.load(preload_file, Loader=yaml.BaseLoader)
+                globals.list_class_name += preload_dict["class"]
             register_nnModule_instance_definition()
             # register transformation
             if feature == "pytorch_dummy_dataloader": # is not in harness scope, but needs call graph and type inference
                 from .coders.pytorch.dummy_dataloader import DummyDataLoader
                 opt = DummyDataLoader(globals.list_model_def_instance)
                 opt.register_transformation()
-            elif feature == "pytorch_reclaim_inputs": # is not in harness scope, but needs call graph and type inference
+            elif feature == "pytorch_reclaim_inputs":
                 from .coders.pytorch.reclaim_inputs import ReclaimInputs
                 opt = ReclaimInputs(globals.list_model_def_instance)
                 opt.register_transformation()
+            elif feature == "pytorch_reclaim_inference_transformers_trainer":
+                from .coders.pytorch.reclaim_inference_transformers_trainer import ReclaimInferenceTransformersTrainer
+                opt = ReclaimInferenceTransformersTrainer(globals.list_model_def_instance)
+                opt.register_transformation()
             elif feature in [
                     "pytorch_inc_dynamic_quant",
                     "pytorch_inc_static_quant_fx",
                     "pytorch_inc_static_quant_ipex",
+                    "onnx_inc_static_quant_qlinear"
                 ]:
                 from .coders.autoinc.autoinc_harness import AutoInc_Harness
                 from .coders.autoinc.calib_dataloader import Calib_Dataloader
@@ -249,6 +280,8 @@ def enable(
             for i in range(len(list_transformed_code)):
                 # Batch Size
                 if "pytorch_change_batch_size" in features:
+                    if "batch_size" in list_transformed_code[0]:  # entry code has "batch_size"
+                        globals.batch_size_changed = True
                     from .coders.pytorch.batch_size import BatchSizeCoder
                     globals.target_batch_size = str(target_batch_size)
                     list_transformed_code[i] = BatchSizeCoder(list_transformed_code[i]).transform()
@@ -267,13 +300,16 @@ def enable(
 
         logger.info(f"Code transformation for feature: [{feature}] finished.")
 
-        for path in globals.list_code_path:
+        for idx_path, path in enumerate(globals.list_code_path):
             if path[-14:] == "_nc_enabled.py":
                 path_transformed = path
             else:
                 path_transformed = path[:-3] + "_nc_enabled.py"
-            open(path_transformed, "w").write(list_transformed_code[globals.list_code_path.index(path)])
-            globals.list_code_path[globals.list_code_path.index(path)] = path_transformed
+            if idx_feature != len(features) - 1:
+                open(path_transformed, "w").write(list_transformed_code[idx_path])
+            else:
+                open(path_transformed, "w").write(list_transformed_code[idx_path].replace(" # [coder-enabled]", ""))
+            globals.list_code_path[idx_path] = path_transformed
         transformed_list_code_path = globals.list_code_path
 
     # test code_line.py
@@ -308,11 +344,7 @@ def enable(
         register_nnModule_instance_definition()
 
     ### Output of Enabling
-    globals.list_code_path, num_user_code_path = handle_user_input.get_all_code_path(
-        code)
-
-    if save_patch_path == "":
-        save_patch_path = ws_path
+    globals.list_code_path, num_user_code_path = handle_user_input.get_all_code_path(code)
 
     if generate_patch:
         whole_patch_user_code = ""
@@ -325,10 +357,12 @@ def enable(
             this_patch, _ = sp_gen_patch.communicate()
             this_patch = str(this_patch)[2:-1]
             whole_patch_user_code += this_patch
-        open(save_patch_path + "neural_coder_patch" + patch_suffix, "w").write(
+        if save_patch_path == "":
+            save_patch_path = ws_path + "neural_coder_patch"
+        open(save_patch_path + patch_suffix, "w").write(
             whole_patch_user_code.replace(r'\n', '\n').replace(r'\t', '\t').replace(r"\'", "\'"))
         abs_patch_path = os.path.abspath(
-            save_patch_path + "neural_coder_patch" + patch_suffix)
+            save_patch_path + patch_suffix)
         logger.info(f"The patch is saved to: [{abs_patch_path}]")
 
         if overwrite:
@@ -348,10 +382,12 @@ def enable(
                 this_patch, _ = sp_gen_patch.communicate()
                 this_patch = str(this_patch)[2:-1]
                 whole_patch_import_modules += this_patch
-            open(save_patch_path + "neural_coder_patch_import_modules" + patch_suffix, "w").write(
+            if save_patch_path == "":
+                save_patch_path = ws_path + "neural_coder_patch_import_modules"
+            open(save_patch_path + patch_suffix, "w").write(
                 whole_patch_import_modules.replace(r'\n', '\n').replace(r'\t', '\t').replace(r"\'", "\'"))
             abs_patch_path = os.path.abspath(
-                save_patch_path + "neural_coder_patch_import_modules" + patch_suffix)
+                save_patch_path + patch_suffix)
             logger.info(
                 f"The patch for imported modules is saved to: [{abs_patch_path}]")
 
@@ -366,10 +402,14 @@ def enable(
 
     ### Benchmark
     if run_bench:
+        if "github.com" in code and ".py" in code:
+            code = globals.list_code_path[0]
+            entry_code = globals.list_code_path[0]
+
         bench_performance, bench_mode, bench_ws_path = bench(
             code=code,
             entry_code=entry_code,
-            entry_code_args=entry_code_args,
+            args=args,
             patch_path=abs_patch_path,
             mode=mode,
             cpu_set_env=cpu_set_env,
@@ -384,7 +424,7 @@ def enable(
 def bench(
     code,
     entry_code="",
-    entry_code_args="",
+    args="",
     patch_path="",
     mode="throughput",  # throughput, latency, multi_instance or self_defined
     logging_level="info",
@@ -508,9 +548,9 @@ def bench(
     bench_log_path = ws_path + "performance.log"
     os.remove(bench_log_path) if os.path.exists(bench_log_path) else 0
 
-    entry_code_args = [entry_code_args]
+    args = [args]
     numa_launcher.exec_launcher(
-        ncore_per_instance, ninstances, entry_code, entry_code_args, bench_log_path)
+        ncore_per_instance, ninstances, entry_code, args, bench_log_path)
 
     # get performance (throughput and latency)
     bench_log = open(bench_log_path, "r", encoding='unicode_escape').read().split('\n')
@@ -566,7 +606,10 @@ def bench(
         IPS[-1] = IPS[-2]
 
     try:
-        FPS = round(sum(IPS) / len(IPS) * ninstances * bench_batch_size, 3)
+        if globals.batch_size_changed: # only times BS if BS has been modified, otherwise times 1
+            FPS = round(sum(IPS) / len(IPS) * ninstances * bench_batch_size, 3)
+        else:
+            FPS = round(sum(IPS) / len(IPS) * ninstances * 1, 3)
     except:
         FPS = 0
     try:
@@ -605,9 +648,10 @@ def bench(
 def superbench(
     code,
     entry_code="",
-    entry_code_args="",
-    sweep_objective="feature",
-    bench_feature=[],
+    args="",
+    sweep_objective="feature",  # "feature" or "bench_config"
+    specify_features=[],
+    bench_feature=[],  # only effective when sweep_objective is "bench_config"
     mode="throughput",
     num_benchmark_iteration=5,
     iteration_dynamic_adjust=True,
@@ -646,11 +690,17 @@ def superbench(
     logger.addHandler(fh)
 
     # print key inputs
-    logger.info(f"Superbench started ...")
-    logger.info(f"code: {code}")
-    logger.info(f"mode: {mode}")
-    logger.info(f"sweep_objective: {sweep_objective}")
-    logger.info(f"num_benchmark_iteration: {num_benchmark_iteration}")
+    if auto_quant:
+        logger.info(f"Auto-Quant started ...")
+        logger.info(f"Code: {code}")
+        logger.info(f"Benchmark Mode: {mode} mode")
+        logger.debug(f"Number of benchmark iterations: {num_benchmark_iteration}")
+    else:
+        logger.info(f"SuperBench started ...")
+        logger.info(f"Code: {code}")
+        logger.info(f"Benchmark Mode: {mode} mode")
+        logger.debug(f"Sweep Objective: {sweep_objective}")
+        logger.debug(f"Number of benchmark iterations: {num_benchmark_iteration}")
 
     # entry code
     if entry_code == "":
@@ -674,36 +724,38 @@ def superbench(
         list_ws_path = []
         result = []
 
-        # features that is a "backend":
-        backends = [
-            "",
-            "pytorch_ipex_fp32",
-            "pytorch_ipex_bf16",
-            "pytorch_inc_static_quant_fx",
-            "pytorch_inc_static_quant_ipex",
-            "pytorch_inc_dynamic_quant",
-            "pytorch_ipex_int8_static_quant",
-            "pytorch_ipex_int8_dynamic_quant",
-        ]
-
-        # features that can be standalone (either use alone or use with "backend"):
-        standalones_pool = [
-            "pytorch_mixed_precision_cpu",
-            "pytorch_channels_last",
-        ]
-
-        if logging_level == "debug" or auto_quant:
-            # features that is a "backend":
+        if auto_quant:
             backends = [
                 [],
-                ["pytorch_inc_bf16"],
                 ["pytorch_inc_dynamic_quant"],
                 ["pytorch_inc_static_quant_fx"],
                 ["pytorch_inc_static_quant_ipex"],
+                ["pytorch_inc_bf16"],
+            ]
+            standalones_pool = []
+        elif len(specify_features) != 0:
+            backends = [
+                [],
+            ]
+            for item in specify_features:
+                backends.append([item])
+            standalones_pool = []
+        else:
+            # features that is a "backend":
+            backends = [
+                "",
+                "pytorch_ipex_fp32",
+                "pytorch_ipex_bf16",
+                "pytorch_inc_static_quant_fx",
+                "pytorch_inc_static_quant_ipex",
+                "pytorch_inc_dynamic_quant",
+                "pytorch_ipex_int8_static_quant",
+                "pytorch_ipex_int8_dynamic_quant",
             ]
-
             # features that can be standalone (either use alone or use with "backend"):
             standalones_pool = [
+                "pytorch_mixed_precision_cpu",
+                "pytorch_channels_last",
             ]
 
         standalones = []
@@ -723,7 +775,9 @@ def superbench(
         for backend in backends:
             for standalone in standalones:
                 features = []
-                if logging_level == "debug":
+                if auto_quant:
+                    features += backend
+                elif len(specify_features) != 0:
                     features += backend
                 else:
                     features.append(backend)
@@ -757,7 +811,7 @@ def superbench(
                 bench_performance, bench_mode, bench_ws_path = enable(
                     code=code,
                     entry_code=entry_code,
-                    entry_code_args=entry_code_args,
+                    args=args,
                     features=features,
                     mode=mode,
                     run_bench=True,
@@ -772,7 +826,7 @@ def superbench(
                     bench_acc, bench_mode, bench_ws_path = enable(
                         code=code,
                         entry_code=entry_code,
-                        entry_code_args=entry_code_args,
+                        args=args,
                         features=features,
                         mode=mode,
                         run_bench=True,
@@ -789,8 +843,8 @@ def superbench(
                     t_end = time.time()
                     if iteration_dynamic_adjust:
                         num_benchmark_iteration = max(int(300 / (t_end - t_start)), 5)
-                    logger.info(
-                        f"Adjusted number of benchmark iterations after dry-run is {num_benchmark_iteration}")
+                        logger.debug(
+                            f"Adjusted number of benchmark iterations after dry-run is {num_benchmark_iteration}")
                     dry_run = False
 
                 def remove_if_have(list, element):
@@ -799,20 +853,43 @@ def superbench(
                     return list
 
                 features = remove_if_have(features, "pytorch_benchmark")
-                features = remove_if_have(
-                    features, "pytorch_change_batch_size")
+                features = remove_if_have(features, "pytorch_change_batch_size")
                 features = remove_if_have(features, "pytorch_cuda_to_cpu")
 
-                if not eval_accuracy:
-                    logger.info(
-                        f"Benchmark result (performance) of optimization set [{features}]"
-                        f" is [{bench_performance[0]}] (FPS)")
+                if auto_quant:
+                    # convert feature name to display name for better user experience
+                    if features == ['pytorch_inc_dynamic_quant']:
+                        features_display = "Intel INT8 (Dynamic)"
+                    elif features == ['pytorch_inc_static_quant_fx']:
+                        features_display = "Intel INT8 (Static)"
+                    elif features == ['pytorch_inc_static_quant_ipex']:
+                        features_display = "Intel INT8 (IPEX)"
+                    elif features == ['pytorch_inc_bf16']:
+                        features_display = "Intel BF16"
+                    elif features == []:
+                        features_display = "The Original Model"
+
+                    if not eval_accuracy:
+                        logger.info(
+                            f"Benchmark result (performance) of {features_display}"
+                            f" is {bench_performance[0]} (FPS)")
+                    else:
+                        logger.info(
+                            f"Benchmark result (performance) of {features_display}"
+                            f" is {bench_performance[0]} (FPS)")
+                        logger.info(
+                            f"Benchmark result (accuracy) of {features_display} is {bench_acc[5]}")
                 else:
-                    logger.info(
-                        f"Benchmark result (performance) of optimization set [{features}]"
-                        f" is [{bench_performance[0]}] (FPS)")
-                    logger.info(
-                        f"Benchmark result (accuracy) of optimization set [{features}] is [{bench_acc[5]}]")
+                    if not eval_accuracy:
+                        logger.info(
+                            f"Benchmark result (performance) of optimization set [{features}]"
+                            f" is [{bench_performance[0]}] (FPS)")
+                    else:
+                        logger.info(
+                            f"Benchmark result (performance) of optimization set [{features}]"
+                            f" is [{bench_performance[0]}] (FPS)")
+                        logger.info(
+                            f"Benchmark result (accuracy) of optimization set [{features}] is [{bench_acc[5]}]")
 
                 d = {}  # initialize dict
                 d["features"] = features
@@ -832,8 +909,7 @@ def superbench(
 
         # print result
         if not eval_accuracy:
-            logger.info(
-                f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
+            print(f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
             print("{:<20} {:<20} {:<120}".format(
                 'Numactl Mode', 'Performance (FPS)', 'Features Applied'))
 
@@ -853,8 +929,7 @@ def superbench(
                         )
                     )
         else:
-            logger.info(
-                f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
+            print(f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
             print("{:<20} {:<20} {:<20} {:<120}".format(
                 'Numactl Mode', 'Performance (FPS)', 'Accuracy', 'Features Applied'))
 
@@ -896,12 +971,42 @@ def superbench(
                     original_model_performance = list_FPS[i]
                     break
         
-        logger.info(f"The best optimization set for your model is: {list_optimization_set_top3[0]}")
-        logger.info(
-            f"You can get up to: "
-            f"{round(list_performance_top3[0] / original_model_performance, 1)}"
-            f" X performance boost with the suggested optimization set."
+        if auto_quant:
+            # convert feature name to display name for better user experience
+            if list_optimization_set_top3[0] == ['pytorch_inc_dynamic_quant']:
+                best_optimization_display = "Intel INT8 (Dynamic)"
+            elif list_optimization_set_top3[0] == ['pytorch_inc_static_quant_fx']:
+                best_optimization_display = "Intel INT8 (Static)"
+            elif list_optimization_set_top3[0] == ['pytorch_inc_static_quant_ipex']:
+                best_optimization_display = "Intel INT8 (IPEX)"
+            elif list_optimization_set_top3[0] == ['pytorch_inc_bf16']:
+                best_optimization_display = "Intel BF16"
+            elif list_optimization_set_top3[0] == []:
+                best_optimization_display = "The Original Model"
+
+            logger.info(f"The best optimization set for your model is {best_optimization_display}")
+            logger.info(
+                f"You can get up to "
+                f"{round(list_performance_top3[0] / original_model_performance, 1)}"
+                f" X performance boost."
+            )
+        else:
+            logger.info(f"The best optimization set for your model is: {list_optimization_set_top3[0]}")
+            logger.info(
+                f"You can get up to "
+                f"{round(list_performance_top3[0] / original_model_performance, 1)}"
+                f" X performance boost."
+            )
+
+        # generate patch for the best optimization
+        features_to_generate = list_optimization_set_top3[0]
+        features_to_generate.append("pytorch_cuda_to_cpu")
+        enable(
+            code=code,
+            features=features_to_generate,
+            save_patch_path="intel_optimization",
         )
+        logger.info('The optimization patch was saved to "intel_optimziation.diff"')
 
         return list_optimization_set_top3, list_performance_top3, original_model_ranking, original_model_performance
 
@@ -950,7 +1055,7 @@ def superbench(
                         bench_performance, bench_mode, bench_ws_path = enable(
                             code=code,
                             entry_code=entry_code,
-                            entry_code_args=entry_code_args,
+                            args=args,
                             features=bench_feature,
                             mode="self_defined",  # sweep bench_config, so mode set to "self_defined"
                             run_bench=True,
@@ -966,8 +1071,9 @@ def superbench(
                             t_end = time.time()
                             if iteration_dynamic_adjust:
                                 num_benchmark_iteration = max(int(300 / (t_end - t_start)), 5)
-                            logger.info(
-                                f"Adjusted number of benchmark iterations after dry-run is {num_benchmark_iteration}")
+                                logger.debug(
+                                    f"Adjusted number of benchmark iterations after dry-run is "
+                                    f"{num_benchmark_iteration}")
                             dry_run = False
 
                         socket_regular_thp = bench_performance[0]
@@ -1115,12 +1221,12 @@ def superbench(
 def auto_quant(
     code,
     entry_code="",
-    entry_code_args="",
+    args="",
     sweep_objective="feature",
     bench_feature=[],
     mode="throughput",
-    num_benchmark_iteration=5,
-    iteration_dynamic_adjust=True,
+    num_benchmark_iteration=30,
+    iteration_dynamic_adjust=False,
     logging_level="info",
     cpu_conversion=True,
     cpu_set_env=True,
@@ -1132,7 +1238,7 @@ def auto_quant(
     return superbench(
         code,
         entry_code=entry_code,
-        entry_code_args=entry_code_args,
+        args=args,
         sweep_objective=sweep_objective,
         bench_feature=bench_feature,
         mode=mode,
