@@ -15,6 +15,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from csv import writer
 import os
 import copy
 import yaml
@@ -43,6 +44,7 @@ class TensorFlowAdaptor(Adaptor):
         "AvgPool": "pooling",
         "ConcatV2": "concat",
         "MatMul": "matmul",
+        "BatchMatMul": "matmul",
         "BatchMatMulV2": "matmul",
         "Pad": "pad"
     }
@@ -55,6 +57,7 @@ class TensorFlowAdaptor(Adaptor):
         self.device = self.framework_specific_info['device']
         self.work_dir = os.path.abspath(self.framework_specific_info['workspace_path'])
         self.recipes = deep_get(self.framework_specific_info, 'recipes', {})
+        self.performance_only = deep_get(self.framework_specific_info, 'performance_only', False)
         os.makedirs(self.work_dir, exist_ok=True)
 
         self.pre_optimized_model = None
@@ -66,7 +69,7 @@ class TensorFlowAdaptor(Adaptor):
 
         cfg_yaml_name = "{}.yaml".format(self.__class__.__name__[:-len('Adaptor')].lower())
         self.query_handler = TensorflowQuery(local_config_file=os.path.join(
-            os.path.dirname(__file__), cfg_yaml_name))
+            os.path.dirname(__file__), cfg_yaml_name), performance_only=self.performance_only)
         self.itex_mode = cfg_yaml_name == 'tensorflow_itex.yaml'
         self.qdq_enabled = cfg_yaml_name == 'inteltensorflow.yaml' or \
                                    cfg_yaml_name == 'tensorflow_itex.yaml'
@@ -85,29 +88,30 @@ class TensorFlowAdaptor(Adaptor):
         values = np.array(values)
 
         # Create histogram using numpy
-        counts, bin_edges = np.histogram(values, bins=bins)
+        # counts, bin_edges = np.histogram(values, bins=bins)
 
         # Fill fields of histogram proto
-        hist = tf.compat.v1.HistogramProto()
-        hist.min = float(np.min(values))
-        hist.max = float(np.max(values))
-        hist.num = int(np.prod(values.shape))
-        hist.sum = float(np.sum(values))
-        hist.sum_squares = float(np.sum(values**2))
+        # hist = tf.compat.v1.HistogramProto()
+        # hist.min = float(np.min(values))
+        # hist.max = float(np.max(values))
+        # hist.num = int(np.prod(values.shape))
+        # hist.sum = float(np.sum(values))
+        # hist.sum_squares = float(np.sum(values**2))
 
-        bin_edges = bin_edges[1:]
+        # bin_edges = bin_edges[1:]
 
-        for edge in bin_edges:
-            hist.bucket_limit.append(edge)
-        for c in counts:
-            hist.bucket.append(c)
+        # for edge in bin_edges:
+        #     hist.bucket_limit.append(edge)
+        # for c in counts:
+        #     hist.bucket.append(c)
 
         # Create and write Summary
-        summary = tf.compat.v1.Summary(value=[tf.compat.v1.Summary.Value(tag=tag, histo=hist)])
-        writer.add_summary(summary, step)
-        writer.flush()
+        # update using TF2.X API
+        with writer.as_default():
+            tf.summary.histogram(tag, values, step)
+            writer.flush()
 
-    def _pre_hook_for_hvd(self):
+    def _pre_hook_for_hvd(self, dataloader=None):
         import horovod.tensorflow as hvd
         self.hvd = hvd
         self.hvd.init()
@@ -273,7 +277,10 @@ class TensorFlowAdaptor(Adaptor):
             if os.path.isdir(temp_dir):
                 import shutil
                 shutil.rmtree(temp_dir, ignore_errors=True)
-            writer = tf.compat.v1.summary.FileWriter(temp_dir, model.graph)
+            # Create the writer using TF2.x APIs to handle eager excutions
+            writer = tf.summary.create_file_writer(temp_dir) # pylint: disable=no-member
+            with writer.as_default():
+                tf.summary.graph(model.graph) # pylint: disable=no-member
 
             cur_graph = GraphAnalyzer()
             cur_graph.graph = model.graph_def
@@ -449,7 +456,6 @@ class TensorFlowAdaptor(Adaptor):
         self.quantize_config['advance'] = deep_get(tuning_cfg, 'advance')
         fp32_ops = []
         bf16_ops = []
-        int8_ops = []
         dispatched_op_names = [j[0] for j in tuning_cfg['op']]
 
         invalid_op_names = [i for i in self.quantize_config['op_wise_config']
@@ -475,7 +481,7 @@ class TensorFlowAdaptor(Adaptor):
             if 'weight' in tuning_cfg['op'][each_op_info]:
                 is_perchannel = tuning_cfg['op'][each_op_info]['weight'][
                     'granularity'] == 'per_channel'
-                bit = tuning_cfg['op'][each_op_info]['weight']['bit']
+                #bit = tuning_cfg['op'][each_op_info]['weight']['bit']
             weight_bit = bit if bit else 7.0
 
             algorithm = tuning_cfg['op'][each_op_info]['activation']['algorithm']
@@ -483,7 +489,6 @@ class TensorFlowAdaptor(Adaptor):
             is_asymmetric = False
             if 'activation' in tuning_cfg['op'][each_op_info]:
                 is_asymmetric = tuning_cfg['op'][each_op_info]['activation']['scheme'] == 'asym'
-            int8_ops.append(op_name)
             self.quantize_config['op_wise_config'][op_name] = (is_perchannel,
                                                                algorithm,
                                                                is_asymmetric,
@@ -543,7 +548,8 @@ class TensorFlowAdaptor(Adaptor):
                                         bf16_ops=self.bf16_ops,
                                         data_loader=data_loader,
                                         qdq_enabled=self.qdq_enabled,
-                                        new_api=self.new_api).convert()
+                                        new_api=self.new_api,
+                                        performance_only = self.performance_only).convert()
             except Exception: # pragma: no cover
                 from .tf_utils.util import get_model_input_shape
                 batch_size = get_model_input_shape(model)
@@ -560,7 +566,8 @@ class TensorFlowAdaptor(Adaptor):
                                         bf16_ops=self.bf16_ops,
                                         data_loader=data_loader,
                                         qdq_enabled=self.qdq_enabled,
-                                        new_api=self.new_api).convert()
+                                        new_api=self.new_api,
+                                        performance_only = self.performance_only).convert()
         else: # pragma: no cover
             if hasattr(data_loader, 'batch_size') and \
               calib_sampling_size % data_loader.batch_size != 0:
@@ -579,7 +586,8 @@ class TensorFlowAdaptor(Adaptor):
                                 bf16_ops=self.bf16_ops,
                                 data_loader=data_loader,
                                 qdq_enabled=self.qdq_enabled,
-                                new_api=self.new_api).convert()
+                                new_api=self.new_api,
+                                performance_only = self.performance_only).convert()
         #just save framework_specific_info feature for recover
         converted_model.q_config.update({'framework_specific_info': \
                                             self.framework_specific_info})
@@ -655,8 +663,10 @@ class TensorFlowAdaptor(Adaptor):
                 node_op = details[-1][0]
                 node_name = details[0]
 
-                self.bf16_op_details[(node_name, node_op)] = {'weight': {'dtype': ['bf16']}, \
-                                                              'activation': {'dtype': ['bf16']}}
+                self.bf16_op_details[(node_name, node_op)] = [{'weight': {'dtype': ['bf16']}, \
+                                                              'activation': {'dtype': ['bf16']}},\
+                                                              {'weight': {'dtype': 'fp32'}, \
+                                                               'activation': {'dtype': 'fp32'}}]
 
     def _query_quantizable_ops(self, matched_nodes):
         """Collect the op-wise configuration for quantization.
@@ -664,6 +674,8 @@ class TensorFlowAdaptor(Adaptor):
         Returns:
             OrderDict: op-wise configuration.
         """
+        bf16_common_config = {'weight': {'dtype': 'bf16'}, 'activation': {'dtype': 'bf16'}}
+        fp32_common_config = {'weight': {'dtype': 'fp32'}, 'activation': {'dtype': 'fp32'}}
         uint8_type = self.query_handler.get_op_types_by_precision(precision='uint8')
         int8_type = self.query_handler.get_op_types_by_precision(precision='int8')
         tf_quantizable_op_type = list(set(uint8_type).union(set(int8_type)))
@@ -674,14 +686,7 @@ class TensorFlowAdaptor(Adaptor):
         conv3d_config = copy.deepcopy(op_capability['uint8']['Conv3D']) if 'Conv3D' in op_capability['uint8'] else None
         matmul_config = copy.deepcopy(op_capability['uint8']['MatMul'])
         other_config = copy.deepcopy(op_capability['uint8']['default'])
-        if ('bf16' in valid_precision and CpuInfo().bf16) or os.getenv('FORCE_BF16') == '1':
-            #TODO we need to enhance below logic by introducing precision priority.
-            conv_config['weight']['dtype'].insert(-1, 'bf16')
-            matmul_config['weight']['dtype'].insert(-1, 'bf16')
-            conv_config['activation']['dtype'].insert(-1, 'bf16')
-            matmul_config['activation']['dtype'].insert(-1, 'bf16')
-            other_config['activation']['dtype'].insert(-1, 'bf16')
-
+        
         self.quantizable_op_details = OrderedDict()
 
         self._init_op_stat = {i: [] for i in tf_quantizable_op_type}
@@ -712,15 +717,14 @@ class TensorFlowAdaptor(Adaptor):
                     conv2d_int8_config['pattern'] = pattern_info
                     self.quantizable_op_details[(
                         node_name, self.unify_op_type_mapping[node_op]
-                    )] = conv2d_int8_config
+                    )] = [conv2d_int8_config, fp32_common_config]
                 elif self.unify_op_type_mapping[node_op].find("conv3d") != -1:
                     conv3d_int8_config = copy.deepcopy(conv3d_config)
                     conv3d_int8_config['pattern'] = pattern_info
                     self.quantizable_op_details[(
                         node_name, self.unify_op_type_mapping[node_op]
-                    )] = conv3d_int8_config
+                    )] = [conv3d_int8_config, fp32_common_config]
                 elif self.unify_op_type_mapping[node_op].find("matmul") != -1:
-
                     matmul_int8_config = copy.deepcopy(matmul_config)
                     matmul_int8_config['pattern'] = pattern_info
                     # TODO enable the sym mode once the tf fixed the mkldequantize_op.cc bug.
@@ -730,11 +734,15 @@ class TensorFlowAdaptor(Adaptor):
                     matmul_int8_config['activation']['scheme'] = matmul_scheme
                     self.quantizable_op_details[(
                         node_name, self.unify_op_type_mapping[node_op]
-                    )] = matmul_int8_config
+                    )] = [matmul_int8_config, fp32_common_config]
                 else:
                     self.quantizable_op_details[(
                         node_name, self.unify_op_type_mapping[node_op]
-                    )] = copy.deepcopy(other_config)
+                    )] = [copy.deepcopy(other_config), fp32_common_config]
+                if ('bf16' in valid_precision and CpuInfo().bf16) or os.getenv('FORCE_BF16') == '1':
+                    self.quantizable_op_details[(
+                        node_name, self.unify_op_type_mapping[node_op]
+                    )].insert(1, bf16_common_config)
 
                 self.quantize_config['op_wise_config'][node_name] = (False, "minmax", False)
         return self.quantizable_op_details
@@ -766,6 +774,30 @@ class TensorFlowAdaptor(Adaptor):
             if not any(input_positive_status):
                 matched_nodes.remove(i)
 
+    def filter_unquantizable_concat_performance_only(self, matched_nodes):
+        target_concat_nodes = [i[0] for i in matched_nodes if i[-1][0] == 'ConcatV2']
+        from neural_compressor.adaptor.tf_utils.util import GraphAnalyzer
+        from neural_compressor.adaptor.tf_utils.graph_util import GraphRewriterHelper
+
+        g = GraphAnalyzer()
+        g.graph = self.pre_optimized_model.graph_def
+        graph_info = g.parse_graph()
+        concat_nodes = g.query_fusion_pattern_nodes([['ConcatV2']])
+        for i in concat_nodes:
+            concat_node_name = i[0]
+            if concat_node_name not in target_concat_nodes:
+                continue
+            input_positive_status = []
+            control_flow = False
+            for index in range(graph_info[concat_node_name].node.attr['N'].i):
+                each_input_name = GraphRewriterHelper.node_name_from_input(
+                    graph_info[concat_node_name].node.input[index])
+                each_input_node = graph_info[each_input_name].node
+                if each_input_node.op in ('Switch'):
+                    control_flow  = True
+            if control_flow:
+                matched_nodes.remove(i)
+
     def query_fw_capability(self, model):
         """Collect the model-wise and op-wise configuration for quantization.
 
@@ -797,7 +829,11 @@ class TensorFlowAdaptor(Adaptor):
                     return True
             return False
 
-        self.filter_unquantizable_concat(matched_nodes)
+
+        if self.new_api and self.performance_only:
+            self.filter_unquantizable_concat_performance_only(matched_nodes)
+        else:
+            self.filter_unquantizable_concat(matched_nodes)
 
         copied_matched_nodes = copy.deepcopy(matched_nodes)
         for i in copied_matched_nodes:
@@ -1293,9 +1329,9 @@ class TensorFlowAdaptor(Adaptor):
         res = OrderedDict()
         for op in self.quantizable_op_details:
             if op[1] not in res:
-                res[op[1]] = {'activation': self.quantizable_op_details[op]['activation']}
-                if 'weight' in self.quantizable_op_details[op]:
-                    res[op[1]]['weight'] = self.quantizable_op_details[op]['weight']
+                    res[op[1]] = {'activation': self.quantizable_op_details[op][0]['activation']}
+                    if 'weight' in self.quantizable_op_details[op][0]:
+                        res[op[1]]['weight'] = self.quantizable_op_details[op][0]['weight']
         for op in self.bf16_op_details:
             if op[1] not in res:
                 res[op[1]] = {'activation': {'dtype': ['bf16']}, 'weight': {'dtype': ['bf16']}}
@@ -1304,7 +1340,8 @@ class TensorFlowAdaptor(Adaptor):
     def _pre_eval_hook(self, model):
         return model
 
-    def _post_eval_hook(self, model):
+    # Add keyword arguments unpacking
+    def _post_eval_hook(self, model, **kwargs):
         pass
 
     def save(self, model, path):
@@ -1323,27 +1360,24 @@ class TensorFlowAdaptor(Adaptor):
 
         quantize_config = {'op_wise_config': {}}
         for each_op_info in capability['opwise']:
-            op_name = each_op_info[0]
-            op_type = each_op_info[1]
-
             is_perchannel = False
             weight_bit = 7.0
-            activation = capability['optypewise'][op_type]['activation']
-            if 'weight' in capability['optypewise'][op_type]:
-                weight = capability['optypewise'][op_type]['weight']
-                is_perchannel = True if weight[
-                    'granularity'][0] == 'per_channel' else False
+            for op_cap in capability['opwise'][each_op_info]:
+                if'activation'in op_cap and 'quant_mode' in op_cap['activation']:
+                    activation = op_cap['activation']
+                    if 'weight' in op_cap:
+                        weight = op_cap['weight']
+                        is_perchannel = True if weight[
+                            'granularity'][0] == 'per_channel' else False
+                    algorithm = activation['algorithm'][0]
+                    is_asymmetric = False
+                    if 'activation' in op_cap:
+                        is_asymmetric = True if activation['scheme'][0] == 'asym' else False
 
-            algorithm = activation['algorithm'][0]
-
-            is_asymmetric = False
-            if 'activation' in capability['optypewise'][op_type]:
-                is_asymmetric = True if activation['scheme'][0] == 'asym' else False
-
-            quantize_config['op_wise_config'][op_name] = (is_perchannel,
-                                                          algorithm,
-                                                          is_asymmetric,
-                                                          weight_bit)
+                    quantize_config['op_wise_config'][each_op_info[0]] = (is_perchannel,
+                                                                          algorithm,
+                                                                          is_asymmetric,
+                                                                          weight_bit)
         from .tf_utils.graph_converter import GraphConverter
         tmp_graphdef = copy.deepcopy(model.graph_def)
         for i in tmp_graphdef.node:
@@ -1353,7 +1387,8 @@ class TensorFlowAdaptor(Adaptor):
         converter = GraphConverter(model,
                                    qt_config=quantize_config,
                                    int8_sequences=self.op_wise_sequences,
-                                   fake_quant=True, new_api=self.new_api)
+                                   fake_quant=True, new_api=self.new_api,
+                                   performance_only = self.performance_only)
 
         return converter.convert()
 
@@ -1377,7 +1412,8 @@ class TensorFlowAdaptor(Adaptor):
         from .tf_utils.graph_converter_without_calib import GraphConverterWithoutCalib
         converter = GraphConverterWithoutCalib(model,
                                             recover_config=q_config,
-                                            new_api=self.new_api)
+                                            new_api=self.new_api,
+                                            performance_only=self.performance_only)
 
         return converter.convert_without_calib()
     
@@ -1438,7 +1474,8 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
                                     data_loader=data_loader,
                                     itex_mode=self.itex_mode,
                                     qdq_enabled=self.qdq_enabled,
-                                    new_api=self.new_api).convert()
+                                    new_api=self.new_api,
+                                    performance_only = self.performance_only).convert()
             except Exception: # pragma: no cover
                 logger.warning(
                         "Fail to forward with batch size={}, set to {} now.".
@@ -1454,7 +1491,8 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
                                 data_loader=data_loader,
                                 itex_mode=self.itex_mode,
                                 qdq_enabled=self.qdq_enabled,
-                                new_api=self.new_api).convert()
+                                new_api=self.new_api,
+                                performance_only = self.performance_only).convert()
         else: # pragma: no cover
             if hasattr(data_loader, 'batch_size') and \
               calib_sampling_size % data_loader.batch_size != 0:
@@ -1474,7 +1512,8 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
                                    data_loader=data_loader,
                                    itex_mode=self.itex_mode,
                                    qdq_enabled=self.qdq_enabled,
-                                   new_api=self.new_api).convert()
+                                   new_api=self.new_api,
+                                   performance_only = self.performance_only).convert()
 
         self._dump_model_op_stats(converted_model.graph_def)
 
@@ -1482,13 +1521,14 @@ class Tensorflow_ITEXAdaptor(TensorFlowAdaptor):
 
 class TensorflowQuery(QueryBackendCapability):
 
-    def __init__(self, local_config_file=None):
+    def __init__(self, local_config_file=None, performance_only=False):
         import tensorflow as tf
 
         super().__init__()
         self.version = tf.version.VERSION
         self.cfg = local_config_file
         self.cur_config = None
+        self.performance_only = performance_only
         self._one_shot_query()
 
     def _get_specified_version_cfg(self, data):
@@ -1549,15 +1589,34 @@ class TensorflowQuery(QueryBackendCapability):
         return config
 
     def _one_shot_query(self):
+        # pylint: disable=E1136
         with open(self.cfg) as f:
             content = yaml.safe_load(f)
             try:
                 self.cur_config = self._get_specified_version_cfg(content)
+                if not self.performance_only:
+                    remove_int8_ops = ['FusedBatchNorm', 'FusedBatchNormV2', 'FusedBatchNormV3']
+                    for op in remove_int8_ops:
+                        while op in self.cur_config['ops']['int8']:
+                            self.cur_config['ops']['int8'].remove(op)
+                        if self.cur_config.get('capabilities'):
+                            self.cur_config['capabilities']['int8'].pop(op, None)
+                        pattern = f'Dequantize + {op} + Relu + QuantizeV2'
+                        if self.cur_config.get('patterns'):
+                            while pattern in self.cur_config['patterns']['int8']:
+                                self.cur_config['patterns']['int8'].remove(pattern)
+
             except Exception as e:
                 logger.info("Fail to parse {} due to {}.".format(self.cfg, str(e)))
                 self.cur_config = None
                 raise ValueError("Please check if the format of {} follows Neural Compressor yaml schema.".
                                  format(self.cfg))
+        self._update_cfg_with_usr_definition()
+
+    def _update_cfg_with_usr_definition(self):
+        from neural_compressor.conf.pythonic_config import tensorflow_config
+        if tensorflow_config.precisions is not None:
+            self.cur_config['precisions']['names'] = ','.join(tensorflow_config.precisions)
 
     def get_version(self):
         """Get the current backend version infomation.
