@@ -49,6 +49,7 @@ from .graph_rewriter.generic.remove_training_nodes import RemoveTrainingNodesOpt
 from .graph_rewriter.generic.strip_unused_nodes import StripUnusedNodesOptimizer
 from .graph_rewriter.generic.fold_batch_norm import FoldBatchNormNodesOptimizer
 from .graph_rewriter.generic.fuse_pad_with_conv import FusePadWithConv2DOptimizer
+from .graph_rewriter.generic.strip_equivalent_nodes import StripEquivalentNodesOptimizer
 from .graph_rewriter.generic.fuse_pad_with_fp32_conv import FusePadWithFP32Conv2DOptimizer
 
 from .graph_rewriter.int8.freeze_value import FreezeValueTransformer
@@ -58,17 +59,21 @@ from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeTran
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeDequantizeTransformer
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeNewAPITransformer
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeDequantizeNewAPITransformer
+from .graph_rewriter.int8.fuse_conv_redundant_dequantize import FuseConvRedundantDequantizeTransformer
+from .graph_rewriter.int8.fuse_matmul_redundant_dequantize import FuseMatMulRedundantDequantizeTransformer
 from .graph_rewriter.int8.scale_propagation import ScaleProPagationTransformer
 from .graph_rewriter.bf16.bf16_convert import BF16Convert
 from .graph_rewriter.int8.post_quantized_op_cse import PostCseOptimizer
+from .graph_rewriter.int8.post_hostconst_converter import PostHostConstConverter
 from .graph_rewriter.int8.meta_op_optimizer import MetaInfoChangingMemOpOptimizer
-from .graph_rewriter.int8.rnn_convert import QuantizedRNNConverter
 from .graph_rewriter.qdq.insert_qdq_pattern import GenerateGraphWithQDQPattern
+from .graph_rewriter.qdq.share_qdq_y_pattern import ShareQDQForItexYPatternOptimizer
+from .graph_rewriter.qdq.merge_duplicated_qdq import MergeDuplicatedQDQOptimizer
 from neural_compressor.adaptor.tf_utils.graph_rewriter.generic.insert_print_node import InsertPrintMinMaxNode
 from .graph_util import GraphRewriterHelper as Helper
 
 
-TF_SUPPORTED_MAX_VERSION = '2.9.1'
+TF_SUPPORTED_MAX_VERSION = '2.10.0'
 TF_SUPPORTED_MIN_VERSION = '1.14.0'
 
 logger = logging.getLogger()
@@ -86,7 +91,8 @@ class GraphConverter:
                  fake_quant=False,
                  itex_mode=False,
                  qdq_enabled=False,
-                 new_api=False):
+                 new_api=False,
+                 performance_only=False):
         """Convert graph.
 
         :param model: input tensorflow model.
@@ -147,7 +153,8 @@ class GraphConverter:
         self._itex_model.input_tensor_names = self.input_tensor_names
         self._tmp_graph_def = copy.deepcopy(self.model.graph_def)
         self.new_api = new_api #bool(version1_gte_version2(tf.version.VERSION, '2.8.0'))
-
+        self.performance_only = performance_only
+        self.exclude_node_names = []
     # pylint: disable=no-member
     def _inference(self, model):
         """Run the calibration on the input graph
@@ -157,6 +164,11 @@ class GraphConverter:
         """
         input_tensor = model.input_tensor
         output_tensor = model.output_tensor
+        # TF table initialization: https://github.com/tensorflow/tensorflow/issues/8665
+        node_names = [node.name for node in model.sess.graph.as_graph_def().node]
+        if 'init_all_tables' in node_names:
+            init_table_op = model.sess.graph.get_operation_by_name('init_all_tables')
+            model.sess.run(init_table_op)
 
         logger.info("Start sampling on calibration dataset.")
         for idx, (inputs, labels) in enumerate(self.data_loader):
@@ -187,7 +199,35 @@ class GraphConverter:
                                 feed_dict[tensor] = inputs[name]
                                 break
                 else:
-                    feed_dict = dict(zip(input_tensor, inputs))
+                    # sometimes the input_tensor is not the same order with inputs
+                    # we should check and pair them
+                    def check_shape(tensor, data):
+                        # scalar or 1 dim default True
+                        if tensor.shape == None or \
+                           len(tensor.shape.dims) == 1 or \
+                           not hasattr(data, 'shape'):
+                            return True
+                        tensor_shape = tuple(tensor.shape)
+                        data_shape = tuple(data.shape)
+                        for tensor_dim, data_dim in zip(tensor_shape, data_shape):
+                            if tensor_dim is not None and tensor_dim != data_dim:
+                                return False
+                        return True
+
+                    disorder_tensors = []
+                    disorder_inputs = [] 
+                    for idx, sort_tensor in enumerate(input_tensor):
+                        sort_input = inputs[idx] 
+                        if check_shape(sort_tensor, sort_input):
+                            feed_dict.update({sort_tensor: sort_input}) 
+                        else:
+                            disorder_tensors.append(sort_tensor)
+                            disorder_inputs.append(sort_input)
+                    for i, dis_tensor in enumerate(disorder_tensors):
+                       for j, dis_input in enumerate(disorder_inputs):  
+                           if check_shape(dis_tensor, dis_input):
+                               feed_dict.update({dis_tensor: dis_input})    
+                               break
             _ = model.sess.run(output_tensor, feed_dict) if model.iter_op==[] \
                 else iterator_sess_run(model.sess, model.iter_op, \
                     feed_dict, output_tensor, self.calib_iteration)
@@ -214,7 +254,7 @@ class GraphConverter:
             if version1_gte_version2(tf.version.VERSION, '2.9.0'):
                 is_supported_version = True
                  
-            if tf.version.VERSION == '1.15.0-up3':
+            if version1_eq_version2(tf.version.VERSION, '1.15.0-up3'):
                 is_supported_version = True
                 
         except Exception as e:
@@ -284,13 +324,26 @@ class GraphConverter:
                 model = self.quantize()
 
         if self.itex_mode:
+            self._itex_model.graph_def = \
+                PostHostConstConverter(self._itex_model.graph_def).do_transformation()
+            self._itex_model.graph_def.library.CopyFrom(self.model.graph_def.library)
             return self._itex_model
-        
-        if len(self.bf16_ops) > 0:
+
+        if self.exclude_node_names:
+            self.bf16_ops.extend(self.exclude_node_names)
+       
+        if (len(self.bf16_ops) > 0 and self.performance_only) or \
+           (os.getenv('MIX_PRECISION_TEST') == '1'):
             model = self.bf16_convert()
+
+        if self.new_api:
+            model.graph_def = FuseConvRedundantDequantizeTransformer(model.graph_def).do_transformation()
+            model.graph_def = FuseMatMulRedundantDequantizeTransformer(model.graph_def).do_transformation()
+
         post_cse_graph_def = PostCseOptimizer(model.graph_def).do_transformation()
-        post_cse_graph_def.library.CopyFrom(self.model.graph_def.library)
-        model.graph_def = post_cse_graph_def
+        post_hostconst_graph_def = PostHostConstConverter(post_cse_graph_def).do_transformation()
+        post_hostconst_graph_def.library.CopyFrom(self.model.graph_def.library)
+        model.graph_def = post_hostconst_graph_def
 
         if debug:
             model.save(self.output_graph)
@@ -366,7 +419,7 @@ class GraphConverter:
         g = GraphAnalyzer()
         g.graph = self._fp32_model.graph_def
         g.parse_graph()
-        y_pattern = [['Conv2D', 'MatMul'], ['BiasAdd'], ['Add'], ('Relu',)]
+        y_pattern = [['Conv2D', 'MatMul'], ['BiasAdd'], ['Add', 'AddV2'], ('Relu',)]
         target_nodes = g.query_fusion_pattern_nodes(y_pattern)
 
         res = {}
@@ -387,11 +440,6 @@ class GraphConverter:
         """
         try:
             self._quantize_graph()
-
-            self._rnn_details = Helper.analysis_rnn_model(self._tmp_graph_def,
-                                                            bf16_ops=self.bf16_ops,
-                                                            fp32_ops=self.fp32_ops)
-            self.quantized_node_info.extend(self._rnn_details.keys())
             self.quantized_node_info = [tuple(i) for i in self.quantized_node_info]
 
             if self.fake_quant:
@@ -407,18 +455,18 @@ class GraphConverter:
                 sampling_graph_def = copy.deepcopy(self._fp32_model.graph_def)
                 # TODO: this is a workaround to make Min/Max node be completly eliminated in int8 graph 
                 # after enabling pad+conv2d in new API.
-                if self.new_api:
-                    non_pad_ops = list(list(set(self.fp32_ops).union(set(self.bf16_ops))))
-                    sampling_graph_def = FusePadWithFP32Conv2DOptimizer(
-                        sampling_graph_def,
-                        non_pad_ops,
-                        self._tmp_model.input_node_names,
-                        self.op_wise_config).do_transformation()
+                
+                non_pad_ops = list(list(set(self.fp32_ops).union(set(self.bf16_ops))))
+                sampling_graph_def = FusePadWithFP32Conv2DOptimizer(
+                    sampling_graph_def,
+                    non_pad_ops,
+                    self._tmp_model.input_node_names,
+                    self.op_wise_config,
+                    self.new_api).do_transformation()
 
                 for i in self.quantized_node_info:
-                    frame_name = self._rnn_details[i] if i in self._rnn_details else None
                     sampling_graph_def, output_names = InsertPrintMinMaxNode(
-                        sampling_graph_def, i[0], i[-1], frame_name).do_transformation()
+                        sampling_graph_def, i[0], i[-1], self.new_api).do_transformation()
                     output_tensor_names.extend(output_names)
                 if self.quantized_node_info:
                     sampling_graph_def.library.CopyFrom(self.model.graph_def.library)
@@ -486,7 +534,7 @@ class GraphConverter:
             self._tmp_model.input_node_names,
             self._tmp_model.output_node_names)
 
-        self._tmp_graph_def, self.quantized_node_info = QuantizeGraphForIntel(
+        self._tmp_graph_def, self.quantized_node_info, exclude_node_names = QuantizeGraphForIntel(
             self._tmp_graph_def,
             self._tmp_model.input_node_names,
             self._tmp_model.output_node_names,
@@ -494,8 +542,10 @@ class GraphConverter:
             self.int8_sequences,
             self.device,
             self.fake_quant,
-            self.new_api).do_transform()
-
+            self.new_api,
+            self.performance_only,
+            self.itex_mode).do_transform()
+        self.exclude_node_names = exclude_node_names
         self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
         if debug:
             self._tmp_model.graph_def = self._tmp_graph_def
@@ -531,11 +581,11 @@ class GraphConverter:
         self._tmp_graph_def, quantizev2_max = FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
-            '__max:').do_transformation()
+            '__max:', device=self.device).do_transformation()
         self._tmp_graph_def, quantizev2_min = FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
-            '__min:').do_transformation()
+            '__min:', device=self.device).do_transformation()
         self._tmp_graph_def, requant_min_max= FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
@@ -548,15 +598,12 @@ class GraphConverter:
         self.scale_info.update(quantizev2_min)
         self.scale_info.update(requant_min_max)
 
-        self._tmp_graph_def = QuantizedRNNConverter(
-            self._tmp_graph_def, self._calibration_data, self._rnn_details, self.new_api).do_transformation()
-
         if 'scale_propagation_max_pooling' in self.recipes and \
                 self.recipes['scale_propagation_max_pooling']:
             self._tmp_graph_def = ScaleProPagationTransformer(
                 self._tmp_graph_def).do_transformation()
 
-        if debug:
+        if debug and not self.new_api:
             self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
             self._tmp_model.graph_def = self._tmp_graph_def
             self._tmp_model.save(self._int8_frozen_range_model_path)
@@ -571,7 +618,6 @@ class GraphConverter:
             self.device, self.new_api).do_transformation()
 
         if not self.fake_quant:
-            # TODO Use MatMul and BatchMatMul new API
             if self.qdq_enabled:
                 self._tmp_graph_def = FuseMatMulRequantizeNewAPITransformer(
                     self._tmp_graph_def).do_transformation()
@@ -584,7 +630,7 @@ class GraphConverter:
 
                 self._tmp_graph_def = FuseMatMulRequantizeDequantizeTransformer(
                             self._tmp_graph_def).do_transformation()
-
+        
         self._tmp_graph_def = StripUnusedNodesOptimizer(
             self._tmp_graph_def,
             self._tmp_model.input_node_names,
@@ -598,13 +644,17 @@ class GraphConverter:
         self._tmp_graph_def = FoldBatchNormNodesOptimizer(
             self._tmp_graph_def).do_transformation()
 
-        if 'scale_propagation_concat' in self.recipes and self.recipes['scale_propagation_concat']:
+        if self.performance_only or ('scale_propagation_concat' in self.recipes \
+             and self.recipes['scale_propagation_concat']):
             self._tmp_graph_def = RerangeQuantizedConcat(self._tmp_graph_def,
-                                                     self.device).do_transformation()
+                self.device, performance_only=self.performance_only).do_transformation()
 
         self._tmp_graph_def = MetaInfoChangingMemOpOptimizer(
             self._tmp_graph_def).do_transformation()
 
+        self._tmp_graph_def = StripEquivalentNodesOptimizer(
+            self._tmp_graph_def, self._tmp_model.output_node_names).do_transformation()
+
         if self.advance_config is not None and \
            deep_get(self.advance_config, 'bias_correction') is not None:
             self._tmp_graph_def = BiasCorrection(
@@ -678,13 +728,9 @@ class GraphConverter:
                                                self.int8_sequences,
                                                self.device,
                                                self.fake_quant,
-                                               self.new_api).get_quantized_nodes()
-
-        self._rnn_details = Helper.analysis_rnn_model(self._tmp_graph_def,
-                                                      bf16_ops=self.bf16_ops,
-                                                      fp32_ops=self.fp32_ops)
-        self.quantized_node_info.extend(self._rnn_details.keys())
-        self.quantized_node_info = [tuple(i) for i in self.quantized_node_info]
+                                               self.new_api,
+                                               self.performance_only,
+                                               self.itex_mode).get_quantized_nodes()
 
         if self.itex_mode:
             self.quantized_node_info.extend(self._search_y_pattern_for_itex())
@@ -705,14 +751,16 @@ class GraphConverter:
                                     sampling_graph_def,
                                     non_pad_ops,
                                     self._tmp_model.input_node_names,
-                                    self.op_wise_config).do_transformation()
+                                    self.op_wise_config,
+                                    self.new_api,
+                                    True).do_transformation()
 
         for i in self.quantized_node_info:
-            frame_name = self._rnn_details[i] if i in self._rnn_details else None
             sampling_graph_def, output_names = InsertPrintMinMaxNode(
-                sampling_graph_def, i[0], i[-1], frame_name).do_transformation()
+                sampling_graph_def, i[0], i[-1], self.new_api).do_transformation()
             output_tensor_names.extend(output_names)
 
+
         if self.quantized_node_info:
             sampling_graph_def.library.CopyFrom(self.model.graph_def.library)
             self._sampling_model.graph_def = sampling_graph_def
@@ -722,32 +770,31 @@ class GraphConverter:
                 self._inference(self._sampling_model)
             self._calibration_data = Helper.gen_valid_sampling_log(tmp_dump_file)
 
-        self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
-        self._tmp_model.graph_def = self._tmp_graph_def
-
         # Insert QDQ pattern
         self._tmp_graph_def = GenerateGraphWithQDQPattern(
-              self._tmp_model, self._calibration_data,
-              self.op_wise_config, self.fake_quant, self.fp32_ops, self.bf16_ops, \
-              self.quantized_node_info, self.device).do_transformation()
+              self._tmp_graph_def, self._calibration_data, self.op_wise_config,
+              self.fake_quant, self.fp32_ops, self.bf16_ops, self.quantized_node_info, 
+              self.device, self.performance_only, self.itex_mode).do_transformation()
 
     def _convert_qdq(self):
         if self.itex_mode:
             self._tmp_graph_def, quantizev2_max = FreezeValueTransformer(
                 self._tmp_graph_def,
                 self._calibration_data,
-                '__max:').do_transformation()
+                '__max:',
+                self.itex_mode).do_transformation()
             self._tmp_graph_def, quantizev2_min = FreezeValueTransformer(
                 self._tmp_graph_def,
                 self._calibration_data,
-                '__min:').do_transformation()
+                '__min:',
+                self.itex_mode).do_transformation()
             self._tmp_graph_def, requant_min_max= FreezeValueTransformer(
                 self._tmp_graph_def,
                 self._calibration_data,
                 '__requant_min_max',
                 tensor_data= self._kl_op_dict,
                 device=self.device,
-                ).do_transformation()
+                itex_mode=self.itex_mode).do_transformation()
 
             self.scale_info.update(quantizev2_max)
             self.scale_info.update(quantizev2_min)
@@ -758,20 +805,25 @@ class GraphConverter:
                 self._tmp_model.input_node_names,
                 self._tmp_model.output_node_names).do_transformation()
 
+            self._tmp_graph_def = ShareQDQForItexYPatternOptimizer(self._tmp_graph_def).do_transformation()
+            self._tmp_graph_def = MergeDuplicatedQDQOptimizer(self._tmp_graph_def).do_transformation()
+
             self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
             self._itex_model.graph_def = self._tmp_graph_def
             self._itex_model.graph_def.library.CopyFrom(self.model.graph_def.library)
         else:
-            self._tmp_graph_def = OptimizeQDQGraph(self._tmp_graph_def,
+            self._tmp_graph_def, exclude_node_names = OptimizeQDQGraph(self._tmp_graph_def,
                                                    self._tmp_model.input_node_names,
                                                    self._tmp_model.output_node_names,
                                                    self.op_wise_config,
                                                    self.int8_sequences,
                                                    self.device,
                                                    self.fake_quant,
-                                                   self.new_api).do_transform()
+                                                   self.new_api,
+                                                   self.performance_only,
+                                                   self.itex_mode).do_transform()
+            self.exclude_node_names=exclude_node_names
 
             if len(self._calibration_data) > 0:
                 self._freeze_requantization_ranges(self._kl_op_dict)
                 self._fuse_requantize_with_fused_quantized_node()
-
