@@ -40,6 +40,7 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
     fuse_sum_op_types = (
         [b'BiasAdd', b'Sum'],
         [b'BiasAdd', b'Sum', b'Relu'],
+        [b'BiasAdd', b'Sum', b'LeakyRelu'],
         [b'BiasAdd', b'Relu', b'Sum'],
         [b'BiasAdd', b'LeakyRelu', b'Sum']
         )
@@ -141,8 +142,6 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
             for _, value in enumerate(quantized_node.input):
                 new_node.input.append(value)
 
-            new_node.input.append(requested_output_min_name)
-            new_node.input.append(requested_output_max_name)
             if 'Tinput' in quantized_node.attr:
                 new_node.attr["Tinput"].CopyFrom(quantized_node.attr['Tinput'])
             if 'Tfilter' in quantized_node.attr:
@@ -157,13 +156,15 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                 new_node.attr["Tsummand"].CopyFrom(quantized_node.attr['Tsummand'])
 
             parent_node_name = Helper.node_name_from_input(quantized_node.input[0])
-            max_filter_node = self.graph_info[new_node.input[6]].node
-            min_filter_node = self.graph_info[new_node.input[5]].node
+            max_filter_node = self.graph_info[new_node.input[-1]].node
+            min_filter_node = self.graph_info[new_node.input[-2]].node
             last_node = self.graph_info[new_node.input[0]].node
-            
+            new_node.input.append(requested_output_min_name)
+            new_node.input.append(requested_output_max_name)
+
             if last_node.op.find('Requantize') != -1 or ((last_node.op.find('QuantizeV2') != -1 or \
                                                             last_node.op.find('QuantizedConv2D') != -1) \
-                                                                and len(quantized_node.attr['fused_ops'].list.s) > 0):          
+                                                                and len(quantized_node.attr['fused_ops'].list.s) > 0):
                 bias_node = self.graph_info[new_node.input[2]].node
                 max_input_node = self.graph_info[last_node.input[-1]].node
                 min_input_node = self.graph_info[last_node.input[-2]].node
@@ -346,14 +347,6 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                         dtypes.float32.as_datatype_enum,
                         dtypes.float32.as_datatype_enum,
                     ])
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
-                    Helper.set_attr_dtype(new_node, "Tsummand", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
                 elif str(quantized_node.attr['fused_ops'].list.s) == str([b"BiasAdd"]):
                     Helper.set_attr_string_list(new_node, 'fused_ops', [b'BiasAdd', b'Requantize'])
                     Helper.set_attr_type_list(new_node, 'Thost_inputs', [
@@ -368,14 +361,6 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                         dtypes.float32.as_datatype_enum,
                         dtypes.float32.as_datatype_enum,
                     ])
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
-                    Helper.set_attr_dtype(new_node, "Tsummand", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
                 elif str(quantized_node.attr['fused_ops'].list.s) == str([b"BiasAdd", b"Relu"]):
                     Helper.set_attr_string_list(new_node, 'fused_ops', [b'BiasAdd', b'Relu', b'Requantize'])
                     Helper.set_attr_type_list(new_node, 'Thost_inputs', [
@@ -390,14 +375,43 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                         dtypes.float32.as_datatype_enum,
                         dtypes.float32.as_datatype_enum,
                     ])
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
-                    Helper.set_attr_dtype(new_node, "Tsummand", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
+                elif str(quantized_node.attr['fused_ops'].list.s) == str([b"BiasAdd", b"LeakyRelu"]):
+                    Helper.set_attr_string_list(new_node, 'fused_ops', [b'BiasAdd', b'LeakyRelu', b'Requantize'])
+                    Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                        input_data_type.as_datatype_enum,
+                        dtypes.qint8.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 else \
+                                                                                    dtypes.qint32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                    ])
+                elif str(quantized_node.attr['fused_ops'].list.s) == str([b"BiasAdd", b"Elu"]):
+                    Helper.set_attr_string_list(new_node, 'fused_ops', [b'BiasAdd', b'Elu', b'Requantize'])
+                    Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                        input_data_type.as_datatype_enum,
+                        dtypes.qint8.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 else \
+                                                                                    dtypes.qint32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                    ])
+
+                Helper.set_attr_type_list(new_node, 'Thost_outputs', [
+                                      requantize_node.attr['out_type'].type,
+                                      dtypes.float32.as_datatype_enum,
+                                      dtypes.float32.as_datatype_enum ])
+                Helper.set_attr_dtype(new_node, "out_type", \
+                    dtype_map_dict[requantize_node.attr['out_type'].type])
+                Helper.set_attr_dtype(new_node, "Tsummand", \
+                    dtype_map_dict[requantize_node.attr['out_type'].type])
                  
             if quantized_node.op == "QuantizedConv2D" or \
                     quantized_node.op == "QuantizedConv2DWithBias" or \
@@ -495,7 +509,7 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
 
             new_node.attr["Tbias"].CopyFrom(attr_value_pb2.AttrValue(type=float32_type))
             new_node.attr["Tsummand"].CopyFrom(attr_value_pb2.AttrValue(type=summand_op_type))
- 
+
             if new_node.op in ('_QuantizedConv2D', '_QuantizedDepthwiseConv2D', '_QuantizedConv3D'):
                 original_input = list(new_node.input)
                 new_input = []
@@ -525,43 +539,28 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                    else dtypes.qint8)
                 if str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'Sum', b'Relu']):
                     self.fused_ops = [b'BiasAdd', b'Sum', b'Relu', b'Requantize']
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
+                elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'Sum', b'LeakyRelu']):
+                    self.fused_ops = [b'BiasAdd', b'Sum', b'LeakyRelu', b'Requantize']
+                elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'LeakyRelu', b'Sum']):
+                    self.fused_ops = [b'BiasAdd', b'LeakyRelu', b'Sum', b'Requantize']
                 elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'Relu', b'Sum']):
                     self.fused_ops = [b'BiasAdd', b'Relu', b'Sum', b'Requantize']
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
                 elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'LeakyRelu', b'Sum']):
                     self.fused_ops = [b'BiasAdd', b'LeakyRelu', b'Sum', b'Requantize']
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
                     #Current fusion requires summand has same dtype as output if output is qint8
                     Helper.set_attr_dtype(new_node, "Tsummand", \
                         dtype_map_dict[requantize_node.attr['out_type'].type])
-
                 elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'Sum']):
                     self.fused_ops = [b'BiasAdd', b'Sum', b'Requantize']
-                    Helper.set_attr_type_list(new_node, 'Thost_outputs', [
-                                          requantize_node.attr['out_type'].type,
-                                          dtypes.float32.as_datatype_enum,
-                                          dtypes.float32.as_datatype_enum ])
-                    Helper.set_attr_dtype(new_node, "out_type", \
-                        dtype_map_dict[requantize_node.attr['out_type'].type])
                     #Current fusion requires summand has same dtype as output if output is qint8
                     Helper.set_attr_dtype(new_node, "Tsummand", \
                         dtype_map_dict[requantize_node.attr['out_type'].type])
+                Helper.set_attr_type_list(new_node, 'Thost_outputs', [
+                                      requantize_node.attr['out_type'].type,
+                                      dtypes.float32.as_datatype_enum,
+                                      dtypes.float32.as_datatype_enum ])
+                Helper.set_attr_dtype(new_node, "out_type", \
+                    dtype_map_dict[requantize_node.attr['out_type'].type])
                 Helper.set_attr_string_list(new_node, 'fused_ops', self.fused_ops)
 
             if not self.new_api:
