@@ -79,12 +79,15 @@ class QuantizeNodeBase():
         self.patterns =  kwargs['patterns']
         self.remove_redundant_quant_flag = kwargs['remove_redundant_quant_flag']
         self.fake_quant = kwargs['fake_quant'] if 'fake_quant' in kwargs else False
+        self.frame_info = kwargs['frame_info'] if 'frame_info' in kwargs else None
         self.per_channel, self.is_asymmetric = kwargs['op_wise_cfg'][0], kwargs['op_wise_cfg'][2]
         self.op_wise_config_name_list = kwargs['op_wise_config_name_list']
         self.weight_bit = kwargs['op_wise_cfg'][3]
         self.start_node_name = kwargs['start_node_name']
         self.device = kwargs['device']
         self.new_api = kwargs['new_api']
+        self.performance_only = kwargs['performance_only']
+        self.itex_mode = kwargs['itex_mode']
         self.enable_s8 = bool(version1_gt_version2(tf.version.VERSION, '2.1.0') or \
                     version1_eq_version2(tf.version.VERSION, '2.1.0') or \
             tf.version.VERSION.find('1.15.0-up') != -1)
@@ -262,15 +265,22 @@ class QuantizeNodeBase():
     def _need_to_check(self, node_type):
         op_list = ("ConcatV2", "Conv2D", "Conv3D", "DepthwiseConv2D", "QuantizeV2", "DepthwiseConv2dNative",
                    "MaxPool", "MaxPool3D", "FusedBatchNormV3", "Requantize", "RequantizePerChannel", "AvgPool", "Pad",
-                   "CropAndResize", "Dequantize", "Mean", "MatMul", "FakeQuantWithMinMaxVars")
+                   "CropAndResize", "Dequantize", "Mean", "MatMul", "BatchMatMulV2", "FakeQuantWithMinMaxVars")
         return any([node_type.find(i) != -1 for i in op_list])
 
     def _find_relu_node(self, node):
         #if node.op.find("HardSwish") != -1:
         #    return False
-        if node.op in ("Relu", "Relu6") or \
+        if (node.op in ("Relu", "Relu6") or \
             (node.op.find("AndRelu") != -1 and \
-            ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0))):
+            ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0)))) \
+                and (node.op != "Relu"
+                     or not self.new_api
+                     or not self.performance_only
+                     or self.node_name_mapping \
+                        [helper.node_name_from_input(node.input[0])].node.op.find("FusedBatchNorm") == -1
+                     or self.node_name_mapping \
+                        [helper.node_name_from_input(node.input[0])].node.attr['is_training'].b):
             return True
         elif 'T' in node.attr and node.attr['T'].type in (dtypes.quint8, dtypes.uint8):
             return True
@@ -395,9 +405,60 @@ class QuantizeNodeBase():
                 all_input_names.append(original_input_name)
         return all_input_names
 
+    def _add_eightbit_prologue_nodes_for_enter(self, original_node, enter_node=None):
+        namespace_prefix = original_node + "_eightbit"
+        reshape_dims_name, reduction_dims_name = self._add_common_quantization_nodes(
+            namespace_prefix, helper.node_name_from_input(
+                self.node_name_mapping[original_node].node.input[0]), enter_node=enter_node)
+        input_names = []
+        min_max_names = []
+        enter_input_name = self.node_name_mapping[original_node].node.input[1]
+        if enter_input_name[0] != '^':
+            if self.node_name_mapping[original_node].node.op == "MatMul":
+                # mkl ops _MklQuantizedMatMulWithBiasAndRelu|AndRequantize
+                # requires the T1 data type as quint8
+                dtype = dtypes.quint8
+            else:
+                input_node_name = helper.node_name_from_input(enter_input_name)
+                if input_node_name in self.output_node_maps:
+                    if self.output_node_maps[input_node_name].op == "Dequantize":
+                        dtype = dtypes.DType(
+                            self.output_node_maps[input_node_name].attr["T"].type)
+                    elif self._find_relu_node(self.node_name_mapping[original_node].node):
+                        dtype = dtypes.quint8
+                    else:
+                        dtype = dtypes.qint8
+                else:
+                    dtype = dtypes.quint8 if self._find_relu_node(
+                        self.node_name_mapping[original_node].node
+                    ) else dtypes.qint8
+                    if 'FusedBatchNorm' in self.node_name_mapping[original_node].node.op:
+                        dtype = dtypes.qint8
+            quantize_input_name, min_input_name, max_input_name = (
+                self._eightbitize_input_to_node(namespace_prefix,
+                                                enter_input_name,
+                                                reshape_dims_name,
+                                                reduction_dims_name,
+                                                dtype=dtype))
+            input_names.append(quantize_input_name)
+            min_max_names.append(min_input_name)
+            min_max_names.append(max_input_name)
+
+        all_input_names = []
+        all_input_names.extend(input_names)
+        if min_max_names:
+            all_input_names.extend(min_max_names)
+
+        for original_input_name in self.node_name_mapping[
+                original_node].node.input:
+            if original_input_name[0] == '^':
+                all_input_names.append(original_input_name)
+        return all_input_names
+
     def _add_common_quantization_nodes(self,
                                        namespace_prefix,
-                                       control_input_names=None):
+                                       control_input_names=None,
+                                       enter_node=None):
         """Builds constant nodes needed for quantization of inputs."""
         reshape_dims_name = namespace_prefix + "_reshape_dims"
         reduction_dims_name = namespace_prefix + "_reduction_dims"
@@ -405,12 +466,35 @@ class QuantizeNodeBase():
         reshape_dims_node = helper.create_constant_node(
             reshape_dims_name, -1, dtypes.int32, [1])
 
+        if enter_node: # pragma: no cover
+            reshape_dims_enter_node = helper.create_node(
+                'Enter', reshape_dims_name+'_enter', [reshape_dims_name])
+            helper.set_attr_string(reshape_dims_enter_node,
+                                   'frame_name', enter_node.attr['frame_name'].s)
+            helper.set_attr_dtype(reshape_dims_enter_node, 'T', dtypes.int32)
+            helper.set_attr_bool(reshape_dims_enter_node, 'is_constant', True)
+            helper.set_attr_int(reshape_dims_enter_node, 'parallel_iterations', \
+                                enter_node.attr['parallel_iterations'].i)
+            self.add_output_graph_node(reshape_dims_enter_node)
+
         self.add_output_graph_node(reshape_dims_node)
         reduction_dims_node = helper.create_constant_node(
             reduction_dims_name, 0, dtypes.int32, [1])
 
+        if enter_node: # pragma: no cover
+            reduction_dims_enter_node = helper.create_node(
+                'Enter', reduction_dims_name+'_enter', [reduction_dims_name])
+            helper.set_attr_string(reduction_dims_enter_node,
+                                   'frame_name', enter_node.attr['frame_name'].s)
+            helper.set_attr_dtype(reduction_dims_enter_node, 'T', dtypes.int32)
+            helper.set_attr_bool(reduction_dims_enter_node, 'is_constant', True)
+            helper.set_attr_int(reduction_dims_enter_node, 'parallel_iterations', \
+                                enter_node.attr['parallel_iterations'].i)
+            self.add_output_graph_node(reduction_dims_enter_node)
+
         self.add_output_graph_node(reduction_dims_node)
-        return reshape_dims_name, reduction_dims_name
+        return reshape_dims_enter_node.name if enter_node else reshape_dims_name, reduction_dims_enter_node.name \
+               if enter_node else reduction_dims_name
 
     def add_output_graph_node(self, output_node):
         """Inserts one node into the new graph."""
@@ -544,7 +628,7 @@ class QuantizeNodeBase():
             # Add a RequantizationRange node for finding the min and max values.
             requant_range_node = helper.create_node(
                 "RequantizationRangePerChannel"
-                if self.per_channel else "RequantizationRange",
+                if self.per_channel or original_node.op == 'DepthwiseConv2dNative' else "RequantizationRange",
                 original_node.name + "_eightbit_requant_range", quantized_outputs)
 
             if self.per_channel:
@@ -578,7 +662,8 @@ class QuantizeNodeBase():
             ]
 
         requantize_node = helper.create_node(
-            "RequantizePerChannel" if self.per_channel else "Requantize",
+            "RequantizePerChannel" if self.per_channel or original_node.op == 'DepthwiseConv2dNative' \
+            else "Requantize",
             original_node.name + "_eightbit_requantize",
             quantized_outputs + min_max_inputs)
         if self.per_channel:
@@ -667,11 +752,23 @@ class QuantizeNodeBase():
     def _intel_cpu_quantize_weight_eightbit(self,
                                             parent,
                                             input_node,
-                                            per_channel):
-        qint8_const_node, min_node, max_node = helper.generate_quantized_weight_node(
-            parent, input_node, per_channel, self.weight_bit, self.device)
-        self.add_output_graph_node(qint8_const_node)
-        self.add_output_graph_node(min_node)
-        self.add_output_graph_node(max_node)
+                                            per_channel,
+                                            enter_node=None):
+        qint8_const_node, min_node, max_node, qint8_const_enter_node, min_enter_node, max_enter_node = \
+            helper.generate_quantized_weight_node(parent, input_node, per_channel,
+                                                  self.weight_bit, self.device, enter_node)
+        if enter_node: # pragma: no cover
+            self.add_output_graph_node(qint8_const_node)
+            self.add_output_graph_node(qint8_const_enter_node)
+            self.add_output_graph_node(min_node)
+            self.add_output_graph_node(min_enter_node)
+            self.add_output_graph_node(max_node)
+            self.add_output_graph_node(max_enter_node)
 
-        return qint8_const_node.name, min_node.name, max_node.name
+            return qint8_const_enter_node.name, min_enter_node.name, max_enter_node.name
+        else:
+            self.add_output_graph_node(qint8_const_node)
+            self.add_output_graph_node(min_node)
+            self.add_output_graph_node(max_node)
+
+            return qint8_const_node.name, min_node.name, max_node.name
