@@ -27,7 +27,7 @@ import re
 import copy
 import itertools
 from collections import OrderedDict
-from .dotdict import DotDict
+from .dotdict import DotDict, deep_set
 import os, datetime
 
 def constructor_register(cls):
@@ -50,16 +50,27 @@ def constructor_register(cls):
 @constructor_register
 class Pruner():
     def __init__(self, start_epoch=None, end_epoch=None, initial_sparsity=None,
-                 target_sparsity=None, update_frequency=1, prune_type='basic_magnitude',
-                 method='per_tensor', names=[], parameters=None):
+                 target_sparsity=None, update_frequency=1,
+                 method='per_tensor',
+                 prune_type='basic_magnitude',##for pytorch pruning, these values should be None
+                 start_step=None, end_step=None, update_frequency_on_step=None, prune_domain=None,
+                 sparsity_decay_type=None, pattern="tile_pattern_1x1", names=None, exclude_names=None, parameters=None):
         self.start_epoch = start_epoch
         self.end_epoch = end_epoch
         self.update_frequency = update_frequency
         self.target_sparsity = target_sparsity
         self.initial_sparsity = initial_sparsity
         self.update_frequency = update_frequency
-        assert prune_type.replace('_', '') in [i.lower() for i in PRUNERS], \
-                                         'now only support {}'.format(PRUNERS.keys())
+        self.start_step = start_step
+        self.end_step = end_step
+        self.update_frequency_on_step = update_frequency_on_step
+        self.prune_domain = prune_domain
+        self.sparsity_decay_type = sparsity_decay_type
+        self.exclude_names = exclude_names
+        self.pattern = pattern
+        ## move this to experimental/pruning to support dynamic pruning
+        # assert prune_type.replace('_', '') in [i.lower() for i in PRUNERS], \
+        #                                  'now only support {}'.format(PRUNERS.keys())
         self.prune_type = prune_type
         self.method = method
         self.names= names
@@ -629,7 +640,7 @@ criterion_schema = Schema({
     },
     Optional('KnowledgeDistillationLoss'): {
         Optional('temperature'): And(float, lambda s: s > 0),
-        Optional('loss_types'): And(list, lambda s: all(i in ['CE', 'KL'] for i in s)),
+        Optional('loss_types'): And(list, lambda s: all(i in ['CE', 'KL', 'MSE'] for i in s)),
         Optional('loss_weights'): And(list, lambda s: all(i >= 0 for i in s) and sum(s) == 1.0),
     },
     Optional('IntermediateLayersKnowledgeDistillationLoss'): {
@@ -638,7 +649,7 @@ criterion_schema = Schema({
         Optional('loss_types'):
             And(Or(tuple, list), lambda s: all(i in ['MSE', 'KL', 'L1'] for i in s)),
         Optional('loss_weights'):
-            And(Or(tuple, list), lambda s: all(i >= 0 for i in s) and sum(s) == 1.0),
+            And(Or(tuple, list), lambda s: all(i >= 0 for i in s)),
         Optional('add_origin_loss'): bool,
     }
 })
@@ -663,15 +674,33 @@ train_schema = Schema({
 weight_compression_schema = Schema({
     Optional('initial_sparsity', default=0): And(float, lambda s: s < 1.0 and s >= 0.0),
     Optional('target_sparsity', default=0.97): float,
+    Optional('max_sparsity_ratio_per_layer', default=0.98):float,
+    Optional('prune_type', default="basic_magnitude"): str,
     Optional('start_epoch', default=0): int,
     Optional('end_epoch', default=4): int,
+    Optional('start_step', default=0): int,
+    Optional('end_step', default=0): int,
+    Optional('update_frequency', default=1.0): float,
+    Optional('update_frequency_on_step', default=1):int,
+    Optional('not_to_prune_names', default=[]):list,
+    Optional('prune_domain', default="global"): str,
+    Optional('names', default=[]): list,
+    Optional('exclude_names', default=None): list,
+    Optional('prune_layer_type', default=None): list,
+    Optional('sparsity_decay_type', default="exp"): str,
+    Optional('pattern', default="tile_pattern_1x1"): str,
+
     Optional('pruners'): And(list, \
                                lambda s: all(isinstance(i, Pruner) for i in s))
 })
 
+# weight_compression_pytorch_schema = Schema({},ignore_extra_keys=True)
+
 approach_schema = Schema({
     Hook('weight_compression', handler=_valid_prune_sparsity): object,
+    Hook('weight_compression_pytorch', handler=_valid_prune_sparsity): object,
     Optional('weight_compression'): weight_compression_schema,
+    Optional('weight_compression_pytorch'): weight_compression_schema,
 })
 
 default_workspace = './nc_workspace/{}/'.format(
@@ -711,6 +740,7 @@ schema = Schema({
             # Now only onnruntime and pytorch supoort
             lambda s: s in ['post_training_static_quant',
                             'post_training_dynamic_quant',
+                            'post_training_auto_quant',
                             'quant_aware_training']),
         Optional('train', default=None): train_schema,
         Optional('advance', default=None): {
@@ -1254,6 +1284,62 @@ class Conf(object):
                 "The yaml file format is not correct. Please refer to document."
             )
 
+    def map_pyconfig_to_cfg(self, pythonic_config):
+        mapping = {
+            'device': pythonic_config.quantization.device,
+            'model.inputs': pythonic_config.quantization.inputs,
+            'model.outputs': pythonic_config.quantization.outputs,
+            'model.framework': pythonic_config.quantization.backend,
+            'quantization.approach': pythonic_config.quantization.approach,
+            'quantization.calibration.sampling_size': 
+                pythonic_config.quantization.calibration_sampling_size,
+            'quantization.optype_wise': pythonic_config.quantization.op_type_list,
+            'quantization.op_wise': pythonic_config.quantization.op_type_list,
+            'distillation.train.criterion': pythonic_config.distillation.criterion,
+            'distillation.train.optimizer': pythonic_config.distillation.optimizer,
+            'pruning.approach.weight_compression': pythonic_config.pruning.weight_compression,
+            'nas.approach': pythonic_config.nas.approach,
+            'nas.search': pythonic_config.nas.search,
+            'nas.dynas': pythonic_config.nas.dynas,
+            'tuning.strategy.name': pythonic_config.quantization.strategy,
+            'tuning.accuracy_criterion.relative': 
+                pythonic_config.quantization.accuracy_criterion.relative,
+            'tuning.accuracy_criterion.absolute':
+                pythonic_config.quantization.accuracy_criterion.absolute,
+            'tuning.accuracy_criterion.higher_is_better':
+                pythonic_config.quantization.accuracy_criterion.higher_is_better,
+            'tuning.objective': pythonic_config.quantization.objective,
+            'tuning.exit_policy.timeout': pythonic_config.quantization.timeout,
+            'tuning.exit_policy.max_trials': pythonic_config.quantization.max_trials,
+            'tuning.exit_policy.performance_only': pythonic_config.quantization.performance_only,
+            'tuning.random_seed': pythonic_config.options.random_seed,
+            'tuning.workspace.path': pythonic_config.options.workspace,
+            'tuning.workspace.resume': pythonic_config.options.resume_from,
+            'evaluation.performance.warmup': pythonic_config.benchmark.warmup,
+            'evaluation.performance.iteration': pythonic_config.benchmark.iteration,
+            'evaluation.performance.pythonic_config.cores_per_instance': 
+                pythonic_config.benchmark.cores_per_instance,
+            'evaluation.performance.pythonic_config.num_of_instance': 
+                pythonic_config.benchmark.num_of_instance,
+            'evaluation.performance.pythonic_config.inter_num_of_threads': 
+                pythonic_config.benchmark.inter_num_of_threads,
+            'evaluation.performance.pythonic_config.intra_num_of_threads': 
+                pythonic_config.benchmark.intra_num_of_threads,
+            'use_bf16': pythonic_config.quantization.use_bf16,
+            'reduce_range': pythonic_config.quantization.reduce_range
+        }
+
+        for k, v in mapping.items():
+            if k in ['tuning.accuracy_criterion.relative', 'tuning.accuracy_criterion.absolute']:
+                target_key = str(pythonic_config.quantization.accuracy_criterion)
+                if target_key not in k and 'accuracy_criterion' in self.usr_cfg.tuning:
+                    if target_key in self.usr_cfg.tuning.accuracy_criterion and \
+                                    k.split('.')[-1] in self.usr_cfg.tuning.accuracy_criterion:
+                        self.usr_cfg.tuning.accuracy_criterion.pop(k.split('.')[-1])
+                    continue
+            if v is not None:
+                deep_set(self.usr_cfg, k, v)
+
     def _convert_cfg(self, src, dst):
         """Helper function to merge user defined dict into default dict.
 
@@ -1355,139 +1441,6 @@ class Quantization_Conf(Conf):
 
         return self._model_wise_tune_space
 
-    def _weight_compute(self, combined_cfg):
-        temp_set = set()
-        for _, config in combined_cfg.items():
-            temp_str = ''
-            for part, params in config.items():
-                temp_str = temp_str + part
-                for _, param in params.items():
-                    temp_str += str(param)
-                temp_str += '_'
-            temp_set.add(temp_str)
-        return len(temp_set)
-
-    def _sort_cfgs(self, combined_cfgs):
-        cfgs_num = len(combined_cfgs)
-        for i in range(cfgs_num):
-            for j in range(cfgs_num-i-1):
-                weight_a = self._weight_compute(combined_cfgs[j])
-                weight_b = self._weight_compute(combined_cfgs[j+1])
-                if weight_a > weight_b:
-                    temp = combined_cfgs[j]
-                    combined_cfgs[j] = combined_cfgs[j+1]
-                    combined_cfgs[j+1] = temp
-        return combined_cfgs
-
-    def _combine_optype_quant_cfgs(self, model_wise_quant_cfgs):
-        if len(model_wise_quant_cfgs) == 0:
-            return []
-        temp_cfgs = OrderedDict()
-
-        for optype, cfgs in model_wise_quant_cfgs.items():
-            if len(cfgs) > 0:
-                temp_cfgs[optype] = copy.deepcopy(cfgs)
-
-        if not bool(temp_cfgs):
-            return []
-
-        keys, values = zip(*temp_cfgs.items())
-
-        return self._sort_cfgs([dict(zip(keys, v)) for v in itertools.product(*values)])
-
-    def opwise_tune_space(self, opwise_quant):
-        def _is_regex(pattern):
-            if re.match("^[A-Za-z0-9.][A-Za-z0-9_.\\-/]*$", pattern):
-                return False
-            return True
-
-        opwise = copy.deepcopy(opwise_quant)
-
-        cfg = self.usr_cfg
-        if cfg.quantization.op_wise:
-            for k, v in cfg.quantization.op_wise.items():
-                is_regex = _is_regex(k)
-                for k_op, _ in opwise.items():
-                    if (not is_regex and k == k_op[0]) or (is_regex and re.match(k, k_op[0])):
-                        opwise[k_op] = self._merge_dicts(v, opwise[k_op])
-
-        for k, v in opwise.items():
-            opwise[k] = self._merge_dicts(self._model_wise_tune_space[k[1]], opwise[k])
-
-        self._opwise_tune_space = opwise
-        return self._opwise_tune_space
-
-    def expand_tune_cfgs(self, tune_space, framework):
-        """generate all possible tuning combinations for each op or model wise tuning.
-
-        Args:
-            tune_space (dict): The tuning space to be expanded.
-
-        Returns:
-            dict: The expanded tuning configs
-        """
-        cfg_lists = self._expand_tune_cfgs_recursively(tune_space)
-
-        # remove unreasonable tuning combinations
-        valid_cfgs = []
-        quant_dtype = ['int8', 'uint8', 'int4', 'uint4']
-
-        for cfg in cfg_lists:
-            cfg = DotDict(cfg)
-            dtype = cfg.activation.dtype
-
-            if dtype not in quant_dtype and "pytorch" not in framework:
-                cfg.activation.clear()
-                cfg.activation.dtype = dtype
-
-            if 'weight' in cfg:
-                dtype = cfg.weight.dtype
-                if dtype not in quant_dtype:
-                    cfg.weight.clear()
-                    cfg.weight.dtype = dtype
-                if (cfg.weight.dtype != cfg.activation.dtype and
-                    cfg.weight.dtype not in quant_dtype and
-                    cfg.activation.dtype not in quant_dtype) or \
-                   (cfg.weight.dtype != cfg.activation.dtype and
-                    cfg.weight.dtype in quant_dtype and
-                    cfg.activation.dtype not in quant_dtype and
-                    ("pytorch" not in framework or
-                     len(tune_space['activation']['dtype']) > 1)) or \
-                   (cfg.weight.dtype != cfg.activation.dtype and
-                    cfg.weight.dtype not in quant_dtype and
-                    cfg.activation.dtype in quant_dtype):
-                    continue
-
-            valid_cfgs.append(cfg)
-
-        # remove duplicated configurations
-        valid_cfgs = [cfg[0] for cfg in itertools.groupby(valid_cfgs)]
-        return valid_cfgs
-
-    def _expand_tune_cfgs_recursively(self, cfg_dict):
-        """Helper function of recursively generating all combinations.
-
-        Args:
-            cfg_dict (dict): The dict of conf space.
-
-        Returns:
-            list: List containing all combinations
-        """
-        assert isinstance(cfg_dict, dict)
-        combinations = OrderedDict()
-        for key in cfg_dict:
-            if isinstance(cfg_dict[key], dict):
-                lists = self._expand_tune_cfgs_recursively(cfg_dict[key])
-                combinations[key] = lists
-
-        if len(combinations) != 0:
-            return self._expand_tune_cfgs_recursively(combinations)
-
-        keys, values = zip(*cfg_dict.items())
-        values = list(filter(None, values))
-        lists = [dict(zip(keys, v)) for v in itertools.product(*values)]
-        return lists
-
 class Pruning_Conf(Conf):
     """config parser.
 
@@ -1498,6 +1451,7 @@ class Pruning_Conf(Conf):
 
     def __init__(self, cfg=None):
         if isinstance(cfg, str):
+            self._read_cfg(cfg)
             self.usr_cfg = DotDict(self._read_cfg(cfg))
         elif isinstance(cfg, DotDict):
             self.usr_cfg = DotDict(schema.validate(self._convert_cfg(
@@ -1615,7 +1569,6 @@ class DefaultConf(DotDict):
     __getattr__ = __getitem__
 
 conf = DefaultConf({})
-
 QuantConf = Quantization_Conf
 PruningConf = Pruning_Conf
 GraphOptConf = Graph_Optimization_Conf
